@@ -307,10 +307,12 @@
 (defun tex-matrix(x l r) ;;matrix looks like ((mmatrix)((mlist) a b) ...)
 ;-  (append l `("\\pmatrix{")
     (append l `(" left( matrix {")
-	  (mapcan #'(lambda(y)
+	  (let ((foo (mapcan #'(lambda(y)
 ;-		      (tex-list (cdr y) nil (list "\\cr ") "&"))
   		      (tex-list (cdr y) nil (list " ## ") " # "))
-		  (cdr x))
+		  (cdr x))))
+            (setf (car (last foo)) " ")
+            foo)
 ;-	  '("}") r))
   	  '("} right) ") r))
 
