@@ -14,7 +14,7 @@
 
 (load-macsyma-macros rzmac)
 
-(declare-top (special opers *a *n $factlim sum msump *i *opers-list opers-list $ratsimpexpons makef))
+(declare-top (special opers *a *n $factlim sum msump *i *opers-list opers-list $ratsimpexpons makef $factorial_expand))
 
 (loop for (x y) on '(%cot %tan %csc %sin %sec %cos %coth %tanh %csch %sinh %sech %cosh)
    by #'cddr do (putprop x y 'recip) (putprop y x 'recip))
@@ -62,7 +62,8 @@
 
 ;; factorial stuff
 
-(setq $factlim -1 makef nil)
+(setq $factlim 100000 ; set to a big integer which will work (not -1)
+      makef nil)
 
 (defmfun $genfact (&rest l)
   (cons '(%genfact) l))
@@ -97,12 +98,52 @@
       1
       (k n 1)))
 
+;;; Factorial has mirror symmetry
+
+(defprop mfactorial t commutes-with-conjugate)
+
 (defmfun simpfact (x y z)
   (oneargcheck x)
   (setq y (simpcheck (cadr x) z))
-  (cond ((or (floatp y) (and (not makef) (ratnump y) (equal (caddr y) 2)))
-	 (simplifya (makegamma1 (list '(mfactorial) y)) nil))
-	(($bfloatp y) (mfuncall '$bffac y $fpprec))
+  (cond ((and (mnump y)
+              (eq ($sign y) '$neg)
+              (zerop1 (sub (simplify (list '(%truncate) y)) y)))
+         ;; Negative integer or a real representation of a negative integer.
+         (merror (intl:gettext "factorial: factorial of negative integer ~:M not defined.") y))
+        ((or (floatp y)             
+             ($bfloatp y)
+             (and (not (integerp y))
+                  (not (ratnump y))
+                  (or (and (complex-number-p y 'float-or-rational-p)
+                           (or $numer 
+                               (floatp ($realpart y)) 
+                               (floatp ($imagpart y))))
+                      (and (complex-number-p y 'bigfloat-or-number-p)
+                           (or $numer
+                               ($bfloatp ($realpart y))
+                               ($bfloatp ($imagpart y))))))
+             (and (not makef) (ratnump y) (equal (caddr y) 2)))
+         ;; Numerically evaluate for real or complex argument in float or
+         ;; bigfloat precision using the Gamma function
+	 (simplify (list '(%gamma) (add 1 y))))
+        ((eq y '$inf) '$inf)
+        ((and $factorial_expand
+              (mplusp y)
+              (integerp (cadr y)))
+         ;; factorial(n+m) and m integer. Expand.
+         (let ((m (cadr y))
+               (n (simplify (cons '(mplus) (cddr y)))))
+           (cond ((>= m 0)
+                  (mul 
+                    (simplify (list '($pochhammer) (add n 1) m))
+                    (simplify (list '(mfactorial) n))))
+                 ((< m 0)
+                  (setq m (- m))
+                  (div
+                    (mul (power -1 m) (simplify (list '(mfactorial) n)))
+                    ;; We factor to get the ordering (n-1)*(n-2)*...
+                    ($factor
+                      (simplify (list '($pochhammer) (mul -1 n) m))))))))
 	((or (not (fixnump y)) (not (> y -1)))
 	 (eqtest (list '(mfactorial) y) x))
 	((or (minusp $factlim) (not (> y $factlim)))
@@ -136,6 +177,19 @@
 	    (take '(%gamma) (sub (add (div x z) 1) y)))))
 	;; End code copied from orthopoly/orthopoly-init.lisp
 
+        ;; Double factorial
+
+        ((eq (caar e) '%factorial_double)
+         (let ((x (makegamma1 (nth 1 e))))
+           (mul
+             (power
+               (div 2 '$%pi)
+               (mul
+                 (div 1 4)
+                 (sub 1 (simplify (list '(%cos) (mul '$%pi x))))))
+             (power 2 (div x 2))
+             (simplify (list '(%gamma) (add 1 (div x 2)))))))
+
 	((eq (caar e) '%elliptic_kc)
 	 ;; Complete elliptic integral of the first kind
 	 (cond ((alike1 (cadr e) '((rat simp) 1 2))
@@ -296,8 +350,15 @@
   (if (not (= (length x) 4)) (wna-err '$genfact))
   (setq z (mapcar #'(lambda (q) (simpcheck q z)) (cdr x)))
   (let ((a (car z)) (b (take '($floor) (cadr z))) (c (caddr z)))
-    (cond ((and (fixnump b) (> b -1)) (gfact a b c))
-	  ((integerp b) (merror "Bad second argument to `genfact': ~:M" b))
+    (cond ((and (fixnump a)
+                (fixnump b)
+                (fixnump c))
+           (if (and (> a -1)
+                    (> b -1) 
+                    (or (<= c a) (= b 0))
+                    (<= b (/ a c)))
+             (gfact a b c)
+             (merror (intl:gettext "genfact: generalized factorial not defined for given arguments."))))
 	  (t (eqtest (list '(%genfact) a
 			   (if (and (not (atom b))
 				    (eq (caar b) '$floor))
@@ -366,7 +427,7 @@ summation when necessary."
 	(ind (cadr l))
 	(lis (meval (caddr l)))
 	(ans 0))
-    (or (symbolp ind) (merror "Second argument not a variable ~M" ind))
+    (or (symbolp ind) (merror (intl:gettext "lsum: second argument must be a variable; found ~M") ind))
     (cond (($listp lis)
 	   (loop for v in (cdr lis)
 	      with lind = (cons ind nil)
@@ -387,7 +448,7 @@ summation when necessary."
 ; some simplifications not handled by the revised code. -- Robert Dodier
 
 (defun simpsum1-save (exp i lo hi)
-  (cond ((not (symbolp i)) (merror "Improper index to `sum':~%~M" i))
+  (cond ((not (symbolp i)) (merror (intl:gettext "sum: index must be a symbol; found ~M") i))
 	((equal lo hi) (mbinding ((list i) (list hi)) (meval exp)))
 	((and (atom exp)
 	      (not (eq exp i))
@@ -425,12 +486,12 @@ summation when necessary."
 (defun dosum (expr ind low hi sump &key (evaluate-summand t))
   (setq low (ratdisrep low) hi (ratdisrep hi)) ;; UGH, GAG WITH ME A SPOON
   (if (not (symbolp ind))
-      (merror "~:M: bad index ~M (must be a symbol)~%" (if sump '$sum '$product) ind))
+      (merror (intl:gettext "~:M: index must be a symbol; found ~M") (if sump '$sum '$product) ind))
   (unwind-protect
        (prog (u *i lind l*i *hl)
 	  (setq lind (cons ind nil))
 	  (cond
-	    ((not (fixnump (setq *hl (m- hi low))))
+	    ((not (fixnump (setq *hl (mfuncall '$floor (m- hi low)))))
 	     (if evaluate-summand (setq expr (mevalsumarg expr ind low hi)))
 	     (return (cons (if sump '(%sum) '(%product))
 			   (list expr ind low hi))))
@@ -478,8 +539,10 @@ summation when necessary."
       (let (($simp nil))
         (setq summand ($substitute gensym-ind ind summand)))
       (setq foo (mbinding ((list gensym-ind) (list gensym-ind))
-			  (resimplify (meval summand))))
-      (let (($simp nil))
+                          (resimplify (meval summand))))
+      ;; At this point we do not switch off simplification to preserve
+      ;; the achieved simplification of the summand (DK 02/2010).
+      (let (($simp t))
 	(setq foo ($substitute ind gensym-ind foo)))
       (if (not (eq hi '$inf))
 	  (forget (list '(mgeqp) hi gensym-ind)))
@@ -496,7 +559,7 @@ summation when necessary."
   (let ((ind (cadr l)))
     (if (mquotep ind) (setq ind (cadr ind)))
     (if (not (symbolp ind))
-      (merror "~:M: bad index ~M (must be a symbol)~%" op ind))
+      (merror (intl:gettext "~:M: index must be a symbol; found ~M") op ind))
     (let ((low (caddr l))
 	  (hi (cadddr l)))
       (list (mevalsumarg (car l) ind low hi)
@@ -521,11 +584,11 @@ summation when necessary."
 
     (setq acc
 	  (cond ((and (eq n '$inf) ($freeof i ex))
-		 (setq sgn ($asksign ex))
+		 (setq sgn (csign ex))
 		 (cond ((eq sgn '$pos) '$inf)
 		       ((eq sgn '$neg) '$minf)
 		       ((eq sgn '$zero) 0)
-		       (t `((%sum simp) ,ex ,lo ,hi))))
+		       (t `((%sum simp) ,ex ,i ,lo ,hi))))
 
 		((and (mbagp e) $listarith)
 		 (simplifya
@@ -611,7 +674,7 @@ summation when necessary."
             
             (t
 	     (setq ex (subst '%product '$product ex))
-	     `((%product) ,(subst k i ex) ,k ,lo ,hi))))
+	     `((%product simp) ,(subst k i ex) ,k ,lo ,hi))))
 
     ;; Hmm, this is curious... don't call existing product simplifications
     ;; if index range is infinite -- what's up with that??
@@ -634,14 +697,14 @@ summation when necessary."
 
 (defun simpprod1-save (exp i lo hi)
   (let (u)
-    (cond ((not (symbolp i)) (merror "Bad index to product:~%~M" i))
+    (cond ((not (symbolp i)) (merror (intl:gettext "product: index must be a symbol; found ~M") i))
 	  ((alike1 lo hi)
 	   (let ((valist (list i)))
 	     (mbinding (valist (list hi))
 		       (meval exp))))
 	  ((eq ($sign (setq u (m- hi lo))) '$neg)
 	   (cond ((eq ($sign (add2 u 1)) '$zero) 1)
-		 (t (merror "Lower bound to product is > upper bound."))))
+		 (t (merror (intl:gettext "product: lower bound ~M greater than upper bound ~M") lo hi))))
 	  ((atom exp)
 	   (cond ((null (eq exp i))
 		  (power* exp (list '(mplus) hi 1 (list '(mtimes) -1 lo))))
@@ -816,7 +879,7 @@ summation when necessary."
 (defun linearize3 (w e x)
   (let (w1)
     (if (and (member w '($inf $minf $infinity) :test #'eq) (safe-zerop x))
-	(merror "Undefined form 0*inf:~%~M" e))
+	(merror (intl:gettext "LINEARIZE3: undefined form 0*inf: ~M") e))
     (setq w (mul2 (simplifya x t) w))
     (cond ((or (atom w) (getl (caar w) '($outative $linear))) (setq w1 1))
 	  ((eq (caar w) 'mtimes)
@@ -843,29 +906,29 @@ summation when necessary."
 	       (return (rplacd l (cddr l))))))))
 
 (defun additive (e z)
-  (cond ((get (caar e) '$outative)	;Really a linearize!
-	 (setq opers-list (copy-list opers-list ))
-	 (rem-opers-p '$outative)
-	 (linearize1 e z))
-	((mplusp (cadr e))
-	 (addn (mapcar #'(lambda (q)
-			   (oper-apply (list* (car e) q (cddr e)) z))
-		       (cdr (cadr e)))
-	       z))
-	(t (oper-apply e z))))
+  (cond ((get (caar e) '$outative)  ; Really a linearize!
+         (setq opers-list (copy-list opers-list))
+         (rem-opers-p '$outative)
+         (linearize1 e z))
+        ((mplusp (cadr e))
+         (addn (mapcar #'(lambda (q)
+                           (let ((opers-list *opers-list))
+                             (oper-apply (list* (car e) q (cddr e)) z)))
+                       (cdr (cadr e)))
+               z))
+        (t (oper-apply e z))))
 
 (setq opers (cons '$multiplicative opers)
       *opers-list (cons '($multiplicative . multiplicative) *opers-list))
 
 (defun multiplicative (e z)
   (cond ((mtimesp (cadr e))
-	 (simptimes
-	  (cons '(mtimes)
-		(mapcar #'(lambda (q)
-			    (oper-apply (list* (car e) q (cddr e)) z))
-			(cdr (cadr e))))
-	  1 z))
-	(t (oper-apply e z))))
+         (muln (mapcar #'(lambda (q)
+                           (let ((opers-list *opers-list))
+                             (oper-apply (list* (car e) q (cddr e)) z)))
+                       (cdr (cadr e)))
+               z))
+        (t (oper-apply e z))))
 
 (setq opers (cons '$outative opers)
       *opers-list (cons '($outative . outative) *opers-list))
@@ -906,13 +969,13 @@ summation when necessary."
 
 (defmfun evenfun (e z)
   (if (or (null (cdr e)) (cddr e))
-      (merror "Even function called with wrong number of arguments:~%~M" e))
+      (merror (intl:gettext "Function declared 'even' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
     (oper-apply (list (car e) (if (mminusp arg) (neg arg) arg)) t)))
 
 (defmfun oddfun (e z)
   (if (or (null (cdr e)) (cddr e))
-      (merror "Odd function called with wrong number of arguments:~%~M" e))
+      (merror (intl:gettext "Function declared 'odd' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
     (if (mminusp arg) (neg (oper-apply (list (car e) (neg arg)) t))
 	(oper-apply (list (car e) arg) t))))
