@@ -5,13 +5,14 @@
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.16.1")
+(defparameter *autoconf-version* "5.22.1")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
-(defparameter *autoconf-host* "x86_64-unknown-openbsd4.7")
+(defparameter *autoconf-host* "sparc64-unknown-openbsd5.0")
 (defparameter *autoconf-win32* "false")
+(defparameter *autoconf-ld-flags* "")
  
-(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.16.1p2/maxima-5.16.1")
+(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.22.1/maxima-5.22.1")
 (defparameter *maxima-default-layout-autotools* "true")
