@@ -61,32 +61,32 @@
 	(t ($at (sdiff (cadr e) x) (caddr e)))))
 
 (defmfun diffncexpt (e x)
-  ((lambda (base* pow)
-     (cond ((and (mnump pow) (or (not (eq (ml-typep pow) 'fixnum)) (< pow 0))) ; POW cannot be 0
-	    (diff%deriv (list e x 1)))
-	   ((and (atom base*) (eq base* x) (free pow base*))
-	    (mul2* pow (list '(mncexpt) base* (add2 pow -1))))
-	   ((ml-typep pow 'fixnum)
-	    ((lambda (deriv ans)
-	       (do ((i 0 (1+ i))) ((= i pow))
-		 (setq ans (cons (list '(mnctimes) (list '(mncexpt) base* i)
-				       (list '(mnctimes) deriv
-					     (list '(mncexpt) base* (- pow 1 i))))
-				 ans)))
-	       (addn ans nil))
-	     (sdiff base* x) nil))
-	   ((and (not (depends pow x)) (or (atom pow) (and (atom base*) (free pow base*))))
-	    ((lambda (deriv index)
-	       (simplifya
-		(list '(%sum)
-		      (list '(mnctimes) (list '(mncexpt) base* index)
-			    (list '(mnctimes) deriv
-				  (list '(mncexpt) base*
-					(list '(mplus) pow -1 (list '(mtimes) -1 index)))))
-		      index 0 (list '(mplus) pow -1)) nil))
-	     (sdiff base* x) (gensumindex)))
-	   (t (diff%deriv (list e x 1)))))
-   (cadr e) (caddr e)))
+  (let ((base* (cadr e))
+	(pow (caddr e)))
+    (cond ((and (mnump pow) (or (not (eq (ml-typep pow) 'fixnum)) (< pow 0))) ; POW cannot be 0
+	   (diff%deriv (list e x 1)))
+	  ((and (atom base*) (eq base* x) (free pow base*))
+	   (mul2* pow (list '(mncexpt) base* (add2 pow -1))))
+	  ((ml-typep pow 'fixnum)
+	   (let ((deriv (sdiff base* x))
+		 (ans nil))
+	     (do ((i 0 (1+ i))) ((= i pow))
+	       (push (list '(mnctimes) (list '(mncexpt) base* i)
+			   (list '(mnctimes) deriv
+				 (list '(mncexpt) base* (- pow 1 i))))
+		     ans))
+	     (addn ans nil)))
+	  ((and (not (depends pow x)) (or (atom pow) (and (atom base*) (free pow base*))))
+	   (let ((deriv (sdiff base* x))
+		 (index (gensumindex)))
+	     (simplifya
+	      (list '(%sum)
+		    (list '(mnctimes) (list '(mncexpt) base* index)
+			  (list '(mnctimes) deriv
+				(list '(mncexpt) base*
+				      (list '(mplus) pow -1 (list '(mtimes) -1 index)))))
+		    index 0 (list '(mplus) pow -1)) nil)))
+	  (t (diff%deriv (list e x 1))))))
 
 (defmfun stotaldiff (e)
   (cond ((or (mnump e) (constant e)) 0)
@@ -105,26 +105,33 @@
 	((eq (caar e) 'mncexpt)
 	 (if (and (ml-typep (caddr e) 'fixnum) (> (caddr e) 0))
 	     (stotaldiff (list '(mnctimes) (cadr e)
-			       (ncpower (cadr e) (f1- (caddr e)))))
+			       (ncpower (cadr e) (1- (caddr e)))))
 	     (list '(%derivative) e)))
 	(t (addn (cons 0 (mapcar #'(lambda (x)
 				     (mul2 (sdiff e x) (list '(%del simp) x)))
 				 (extractvars (margs e))))
 		 t))))
 
-(defun extractvars (e)
+(defun extractvars (e &aux vars)
   (cond ((null e) nil)
 	((atom (car e))
-	 (if (not (maxima-constantp (car e)))
-	     (union* (ncons (car e)) (extractvars (cdr e)))
-	     (extractvars (cdr e))))
-	((member 'array (cdaar e) :test #'eq) (union* (ncons (car e)) (extractvars (cdr e))))
+	 (cond ((not (maxima-constantp (car e)))
+	        (cond ((setq vars (mget (car e) 'depends))
+	               ;; The symbol has dependencies. Put the dependencies on
+	               ;; the list of extracted vars.
+	               (union* vars (extractvars (cdr e))))
+	              (t
+	               ;; Put the symbol on the list of extracted vars.
+	               (union* (ncons (car e)) (extractvars (cdr e))))))
+	     (t (extractvars (cdr e)))))
+	((member 'array (cdaar e) :test #'eq)
+	 (union* (ncons (car e)) (extractvars (cdr e))))
 	(t (union* (extractvars (cdar e)) (extractvars (cdr e))))))
 
 ;;;; AT
 
 ;;dummy-variable-operators is defined in COMM, which uses it inside of SUBST1.
-(declare-top (special atvars ateqs atp munbound dummy-variable-operators))
+(declare-top (special atvars ateqs *atp* munbound dummy-variable-operators))
 
 (defmfun $atvalue (exp eqs val)
   (let (dl vl fun)
@@ -139,7 +146,7 @@
     (if (or (mopp fun) (eq fun 'mqapply)) (improper-arg-err exp '$atvalue))
     (atvarschk vl)
     (do ((vl1 vl (cdr vl1)) (l atvars (cdr l))) ((null vl1))
-      (if (and (symbolp (car vl1)) (not (mget (car vl1) '$constant)))
+      (if (and (symbolp (car vl1)) (not (kindp (car vl1) '$constant)))
 	  (setq val (maxima-substitute (car l) (car vl1) val))
 	  (improper-arg-err (cons '(mlist) vl) '$atvalue)))
     (setq eqs (if (eq (caar eqs) 'mequal) (list eqs) (cdr eqs)))
@@ -158,64 +165,87 @@
     (add2lnc fun $props)
     val))
 
-(defmfun $at (exp ateqs)
+(defprop %at simp-%at operators)
+
+(defun simp-%at (expr ignored simp-flag)
+  (declare (ignore ignored))
+  (twoargcheck expr)
+  (let ((arg (simpcheck (cadr expr) simp-flag))
+        (eqn (if ($listp (caddr expr))
+                 (cons '(mlist simp) (cdr ($sort (caddr expr))))
+                 (caddr expr))))
+    (cond (($constantp arg) arg)
+          ((alike1 eqn '((mlist))) arg)
+          (t (eqtest (list '(%at) arg eqn) expr)))))
+
+(defmfun $at (expr ateqs)
   (if (notloreq ateqs) (improper-arg-err ateqs '$at))
-  (atscan (let ((atp t)) ($substitute ateqs exp))))
+  (atscan (let ((*atp* t)) ($psubstitute ateqs expr))))
 
-(defun atscan (exp)
-  (cond ((or (atom exp) (member (caar exp) '(%at mrat) :test #'eq) (like ateqs '((mlist)))) exp)
-	((eq (caar exp) '%derivative)
-	 (or (and (not (atom (cadr exp)))
-		  (let ((vl (cdadr exp)) dl)
-		    (dolist (v vl)
-		      (setq dl (nconc dl (ncons (or (getf (cddr exp) v)
-						    0)))))
-		    (atfind (caaadr exp)
-			    (cdr ($substitute ateqs (cons '(mlist) vl)))
-			    dl)))
-	     (list '(%at) exp ateqs)))
-	((member (caar exp) dummy-variable-operators :test #'eq) (list '(%at) exp ateqs))
-	((at1 exp))
-	(t (recur-apply #'atscan exp))))
+(defun atscan (expr)
+  (cond ((or (atom expr)
+             (eq (caar expr) 'mrat)
+             (like ateqs '((mlist))))
+         expr)
+        ((eq (caar expr) '%derivative)
+         (or (and (not (atom (cadr expr)))
+                  (let ((vl (cdadr expr)) dl)
+                    (dolist (v vl)
+                      (setq dl (nconc dl (ncons (or (getf (cddr expr) v) 0)))))
+                    (atfind (caaadr expr)
+                            (cdr ($psubstitute ateqs (cons '(mlist) vl)))
+                            dl)))
+             (list '(%at) expr ateqs)))
+        ((member (caar expr) dummy-variable-operators :test #'eq)
+         (list '(%at) expr ateqs))
+        ((at1 expr))
+        (t (recur-apply #'atscan expr))))
 
-(defun at1 (exp) (atfind (caar exp) (cdr exp) (listof0s (cdr exp))))
+(defun at1 (expr)
+  (atfind (caar expr) (cdr expr) (listof0s (cdr expr))))
 
 (defun atfind (fun vl dl)
-  (do ((atvalues (mget fun 'atvalues) (cdr atvalues))) ((null atvalues))
+  (do ((atvalues (mget fun 'atvalues) (cdr atvalues)))
+      ((null atvalues))
     (and (equal (caar atvalues) dl)
 	 (do ((l (cadar atvalues) (cdr l)) (vl vl (cdr vl)))
 	     ((null l) t)
 	   (if (and (not (equal (car l) (car vl)))
 		    (not (eq (car l) munbound)))
 	       (return nil)))
-	 (return (prog2 (atvarschk vl)
-		     (substitutel vl atvars (caddar atvalues)))))))
+         (return (prog2
+                    (atvarschk vl)
+                    (substitutel vl atvars (caddar atvalues)))))))
 
 (defun listof0s (llist)
-  (do ((llist llist (cdr llist)) (l nil (cons 0 l))) ((null llist) l)))
+  (do ((llist llist (cdr llist)) (l nil (cons 0 l)))
+      ((null llist) l)))
 
 (declare-top (special $ratfac genvar varlist $keepfloat *e*))
 
-
 (defmvar $logconcoeffp nil)
 (defmvar superlogcon t)
 (defmvar $superlogcon t)
 
-(defmfun $logcontract (e) (lgccheck (logcon e))) ; E is assumed to be simplified.
+(defmfun $logcontract (e)
+  (lgccheck (logcon e))) ; E is assumed to be simplified.
 
 (defun logcon (e)
   (cond ((atom e) e)
 	((member (caar e) '(mplus mtimes) :test #'eq)
 	 (if (and $superlogcon (not (lgcsimplep e))) (setq e (lgcsort e)))
-	 (cond ((mplusp e) (lgcplus e)) ((mtimesp e) (lgctimes e)) (t (logcon e))))
+         (cond ((mplusp e) (lgcplus e))
+               ((mtimesp e) (lgctimes e))
+               (t (logcon e))))
 	(t (recur-apply #'logcon e))))
 
 (defun lgcplus (e)
   (do ((x (cdr e) (cdr x)) (log) (notlogs) (y))
       ((null x)
        (cond ((null log) (subst0 (cons '(mplus) (nreverse notlogs)) e))
-	     (t (setq log (sratsimp (muln log t)))
-		(addn (cons (lgcsimp log) notlogs) t))))
+             (t
+              (setq log (let (($ratfac t)) (sratsimp (muln log t))))
+              (addn (cons (lgcsimp log) notlogs) t))))
     (cond ((atom (car x)) (setq notlogs (cons (car x) notlogs)))
 	  ((eq (caaar x) '%log) (setq log (cons (logcon (cadar x)) log)))
 	  ((eq (caaar x) 'mtimes)
@@ -241,7 +271,14 @@
 		   (t (setq notlogs (cons (car x) notlogs))))))))
 
 (defun lgcsimp (e)
-  (cond ((atom e) (simpln (list '(%log) e) 1 t)) (t (list '(%log simp) e))))
+  (cond ((atom e)
+         ;; e.g. log(1) -> 0, or log(%e) -> 1
+         (simplify (list '(%log) e)))
+        ((and (mexptp e) (eq (cadr e) '$%e))
+         ;; log(%e^expr) -> expr
+         (simplify (list '(%log) e)))
+        (t
+         (list '(%log simp) e))))
 
 (defun lgcsimplep (e)
   (and (eq (caar e) 'mplus)
@@ -281,7 +318,6 @@
 		 (list '(%log simp) (if (= (car num) 1) denom (neg denom)))))
 	  (t (recur-apply #'lgccheck e)))))
 
-
 (defun logconcoeffp (e)
   (if $logconcoeffp (let ((*e* e)) (is '(($logconcoeffp) *e*))) (maxima-integerp e)))
 
@@ -292,20 +328,29 @@
 (defmvar $rootsconmode t)
 
 (defun $rootscontract (e)	       ; E is assumed to be simplified
-  ((lambda (radpe $radexpand) (rtcon e))
-   (and $radexpand (not (eq $radexpand '$all)) (eq $domain '$real)) nil))
+  (let ((radpe (and $radexpand (not (eq $radexpand '$all)) (eq $domain '$real)))
+	($radexpand nil))
+    (rtcon e)))
 
 (defun rtcon (e)
   (cond ((atom e) e)
 	((eq (caar e) 'mtimes)
-	 (if (and (not (free e '$%i))
-		  (let ((num ($num e)))
-		    (and (not (alike1 e num))
-			 (or (eq num '$%i)
-			     (and (not (atom num)) (member '$%i num :test #'eq)
-				  (member '$%i (rtcon num) :test #'eq))))))
-	     (setq e (list* (car e) -1 '((mexpt) -1 ((rat simp) -1 2))
-			    (delete '$%i (copy-list (cdr e)) :count 1 :test #'eq))))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+; This code does the substitution %i -> (-1)*(-1)^(-1/2) for expressions
+; like %i/expr. In most cases this causes a wrong result, e.g. 
+; bug ID: 2820202 - rootscontract(%i/2) --> -%i/2. The only case this codes 
+; works is for expressions like %i/sqrt(x) --> -1/sqrt(x). It is doubtful that 
+; this simplification is useful.
+; To get correct results in general, this code is commented out. DK 08/2009.
+; 
+;	 (if (and (not (free e '$%i))
+;		  (let ((num ($num e)))
+;		    (and (not (alike1 e num))
+;			 (or (eq num '$%i)
+;			     (and (not (atom num)) (member '$%i num :test #'eq)
+;				  (member '$%i (rtcon num) :test #'eq))))))
+;	     (setq e (list* (car e) -1 '((mexpt) -1 ((rat simp) -1 2))
+;			    (delete '$%i (copy-list (cdr e)) :count 1 :test #'eq))))
 	 (do ((x (cdr e) (cdr x)) (roots) (notroots) (y))
 	     ((null x)
 	      (cond ((null roots) (subst0 (cons '(mtimes) (nreverse notroots)) e))
@@ -397,59 +442,79 @@
 
 ;;;; ATAN2
 
-(declare-top (special $numer $%piargs $logarc $trigsign half%pi fourth%pi))
+(declare-top (special $numer $logarc $trigsign))
 
-(defun simpatan2 (e vestigial z)	; atan2(y,x) ~ atan(y/x)
+(defun simpatan2 (expr vestigial z)     ; atan2(y,x) ~ atan(y/x)
   (declare (ignore vestigial))
-  (twoargcheck e)
+  (twoargcheck expr)
   (let (y x signy signx)
-    (setq y (simpcheck (cadr e) z) x (simpcheck (caddr e) z))
+    (setq y (simpcheck (cadr expr) z)
+          x (simpcheck (caddr expr) z))
     (cond ((and (zerop1 y) (zerop1 x))
-	   (merror "atan2(0,0) has been generated."))
-	  ( ;; float contagion
-	   (and (or (numberp x) (ratnump x)) ; both numbers
-		(or (numberp y) (ratnump y)) ; ...but not bigfloats
-		(or $numer (floatp x) (floatp y))) ;at least one float
-	   (atan2 ($float y) ($float x)))
-	  ( ;; bfloat contagion
-	   (and (mnump x)
-		(mnump y)
-		(or ($bfloatp x) ($bfloatp y))) ;at least one bfloat
-	   (setq x ($bfloat x)
-		 y ($bfloat y))
-	   (*fpatan y (list x)))
-	  ((and $%piargs (free x '$%i) (free y '$%i)
-		;; Only use asksign if %piargs is on.
-		(cond ((zerop1 y) (if (atan2negp x) (simplify '$%pi) 0))
-		      ((zerop1 x)
-		       (if (atan2negp y) (mul2* -1 half%pi) (simplify half%pi)))
-		      ((alike1 y x)
-		       ;; Should we check if ($sign x) is $zero here?
-		       (if (atan2negp x) (mul2* -3 fourth%pi) (simplify fourth%pi)))
-		      ((alike1 y (mul2 -1 x))
-		       (if (atan2negp x) (mul2* 3 fourth%pi) (mul2* -1 fourth%pi)))
-		      ;; Why is atan2(1,sqrt(3)) super-special-cased here?!?!
-		      ;; It doesn't even handle atan2(1,-sqrt(3));
-		      ;; *Atan* should handle sqrt(3) etc., so all cases will work
-		      ((and (equal y 1) (alike1 x '((mexpt simp) 3 ((rat simp) 1 2))))
-		       (mul2* '((rat simp) 1 6) '$%pi)))))
-	  ($logarc (logarc '%atan (div y x)))
-	  ((and $trigsign (mminusp* y))
-	   (neg (simplifya (list '($atan2) (neg y) x) t)))
-					; atan2(y,x) = atan(y/x) + pi sign(y) (1-sign(x))/2
-	  ((and (free x '$%i) (eq (setq signx ($sign x)) '$pos))
-	   (simplifya (list '(%atan) (div y x)) t))
-	  ((and (eq signx '$neg) (free y '$%i)
-		(member (setq signy ($sign y)) '($pos $neg) :test #'eq))
-	   (add2 (simplifya (list '(%atan) (div y x)) t)
-		 (porm (eq signy '$pos) (simplify '$%pi))))
-	  ((and (eq signx '$zero) (eq signy '$zero))
-	   ;; Unfortunately, we'll rarely get here.  For example,
-	   ;; assume(equal(x,0)) atan2(x,x) simplifies via the alike1 case above
-	   (merror "atan2(0,0) has been generated."))
-	  (t (eqtest (list '($atan2) y x) e)))))
-
-(defun atan2negp (e) (eq (asksign-p-or-n e) '$neg))
+           (merror (intl:gettext "atan2: atan2(0,0) is undefined.")))
+          ( ;; float contagion
+           (and (or (numberp x) (ratnump x))       ; both numbers
+                (or (numberp y) (ratnump y))       ; ... but not bigfloats
+                (or $numer (floatp x) (floatp y))) ; at least one float
+           (atan2 ($float y) ($float x)))
+          ( ;; bfloat contagion
+           (and (mnump x)
+                (mnump y)
+                (or ($bfloatp x) ($bfloatp y)))    ; at least one bfloat
+           (setq x ($bfloat x)
+                 y ($bfloat y))
+           (*fpatan y (list x)))
+          ;; Simplifify infinities
+          ((or (eq x '$inf)
+               (alike1 x '((mtimes) -1 $minf)))
+           ;; Simplify atan2(y,inf) -> 0
+           0)
+          ((or (eq x '$minf)
+               (alike1 x '((mtimes) -1 $inf)))
+           ;; Simplify atan2(y,minf) -> %pi for realpart(y)>=0 or 
+           ;; -%pi for realpart(y)<0. When sign of y unknwon, return noun form.
+           (cond ((member (setq signy ($sign ($realpart x))) '($pos $pz $zero)) 
+                  '$%pi)
+                 ((eq signy '$neg) (mul -1 '$%pi))
+                 (t (eqtest (list '($atan2) y x) expr))))
+          ((or (eq y '$inf)
+               (alike1 y '((mtimes) -1 $minf)))
+           ;; Simplify atan2(inf,x) -> %pi/2
+           (div '$%pi 2))
+          ((or (eq y '$minf)
+               (alike1 y '((mtimes -1 $inf))))
+           ;; Simplify atan2(minf,x) -> -%pi/2
+           (div '$%pi -2))
+          ((and (free x '$%i) (setq signx ($sign x))
+                (free y '$%i) (setq signy ($sign y))
+                (cond ((zerop1 y)
+                       (cond ((eq signx '$neg) '$%pi)
+                             ((member signx '($pos $pz)) 0)))
+                      ((zerop1 x)
+                       (cond ((eq signy '$neg) (div '$%pi -2))
+                             ((member signy '($pos $pz)) (div '$%pi 2))))
+                      ((alike1 y x)
+                       (cond ((eq signx '$neg) (mul -3 (div '$%pi 4)))
+                             ((member signx '($pos $pz)) (div '$%pi 4))))
+                      ((alike1 y (mul -1 x))
+                       (cond ((eq signx '$neg) (mul 3 (div '$%pi 4)))
+                             ((member signx '($pos $pz)) (div '$%pi -4)))))))
+          ($logarc
+           (logarc '%atan2 (list ($logarc y) ($logarc x))))
+          ((and $trigsign (mminusp* y))
+           (neg (take '($atan2) (neg y) x)))
+          ;; atan2(y,x) = atan(y/x) + pi sign(y) (1-sign(x))/2
+          ((eq signx '$pos)
+           (take '(%atan) (div y x)))
+          ((and (eq signx '$neg)
+                (member (setq signy ($csign y)) '($pos $neg) :test #'eq))
+           (add (take '(%atan) (div y x))
+                (porm (eq signy '$pos) '$%pi)))
+          ((and (eq signx '$zero) (eq signy '$zero))
+           ;; Unfortunately, we'll rarely get here.  For example,
+           ;; assume(equal(x,0)) atan2(x,x) simplifies via the alike1 case above
+           (merror (intl:gettext "atan2: atan2(0,0) is undefined.")))
+          (t (eqtest (list '($atan2) y x) expr)))))
 
 ;;;; ARITHF
 
@@ -466,16 +531,15 @@
 
 (defmspec $numerval (l) (setq l (cdr l))
 	  (do ((l l (cddr l)) (x (ncons '(mlist simp)))) ((null l) x)
-	    (cond ((null (cdr l)) (merror "`numerval' takes an even number of args"))
+	    (cond ((null (cdr l)) (merror (intl:gettext "numerval: expected an even number of arguments.")))
 		  ((not (symbolp (car l)))
-		   (merror "~M must be atomic - `numerval'" (car l)))
+		   (merror (intl:gettext "numerval: expected a symbol; found ~M") (car l)))
 		  ((boundp (car l))
-		   (merror "~M is bound - `numerval'" (car l))))
+		   (merror (intl:gettext "numerval: cannot declare a value because ~M is bound.") (car l))))
 	    (mputprop (car l) (cadr l) '$numer)
 	    (add2lnc (car l) $props)
 	    (nconc x (ncons (car l)))))
 
-
 (declare-top (special powers var depvar))
 
 (defmfun $derivdegree (e depvar var)
@@ -494,6 +558,10 @@
 
 ;;;; BOX
 
+;; Set the the property reversealias
+(defprop mbox $box reversealias)
+(defprop mlabox $box reversealias)
+
 (defmfun $dpart (&rest args)
   (mpart args nil t nil '$dpart))
 
@@ -543,10 +611,10 @@
 (defmfun scanmap1 (func e &optional (flag nil flag?))
   (let ((arg2 (specrepcheck e)) newarg2)
     (cond ((eq func '$rat)
-	   (merror "`scanmap' results must be in general representation."))
+	   (merror (intl:gettext "scanmap: cannot apply 'rat'.")))
 	  (flag?
 	   (unless (eq flag '$bottomup)
-	     (merror "Only `bottomup' is an acceptable 3rd arg to `scanmap'."))
+	     (merror (intl:gettext "scanmap: third argument must be 'bottomup', if present; found ~M") flag))
 	   (if (mapatom arg2)
 	       (funcer func (ncons arg2))
 	       (subst0 (funcer func
@@ -600,8 +668,11 @@
 		#'(lambda (i j) (meval (list (list a 'array) i j)))
 	      #'(lambda (i j) (mfuncall a i j))))
     
-    (if (or (notevery #'fixnump (list i2 j2 i1 j1)) (> i1 i2) (> j1 j2))
-	(merror "Invalid arguments to `genmatrix':~%~M" (list '(mlist) a i2 j2 i1 j1)))
+    (if (notevery #'fixnump (list i2 j2 i1 j1))
+      (merror (intl:gettext "genmatrix: bounds must be integers; found ~M, ~M, ~M, ~M") i2 j2 i1 j1))
+ 	 
+    (if (or (> i1 i2) (> j1 j2))
+      (merror (intl:gettext "genmatrix: upper bounds must be greater than or equal to lower bounds; found ~M, ~M, ~M, ~M") i2 j2 i1 j1))
  	 
     (dotimes (i (1+ (- i2 i1)))
       (nconc l (ncons (ncons '(mlist)))))
@@ -619,12 +690,12 @@
 
 (defmfun $copymatrix (x)
   (unless ($matrixp x)
-    (merror "Argument not a matrix - `copymatrix':~%~M" x))
+    (merror (intl:gettext "copymatrix: argument must be a matrix; found ~M") x))
   (copy-tree x))
 
 (defmfun $copylist (x)
   (unless ($listp x)
-    (merror "Argument not a list - `copylist':~%~M" x))
+    (merror (intl:gettext "copylist: argument must be a list; found ~M") x))
   (copy-tree x))
 
 (defmfun $copy (x)
@@ -634,35 +705,40 @@
 
 (defmfun $addrow (m &rest rows)
   (declare (dynamic-extent rows))
-  (cond ((not ($matrixp m)) (merror "First argument to `addrow' must be a matrix"))
-	((null rows) m)
-	(t (dolist (r rows m)
-	     (setq m (addrow m r))))))
+  (cond ((not ($matrixp m))
+         (merror
+           (intl:gettext "addrow: first argument must be a matrix; found ~M")
+          m))
+        ((null rows) m)
+        (t
+         (let ((m (copy-tree m)))
+           (dolist (r rows m)
+             (setq m (addrow m r)))))))
 
 (defmfun $addcol (m &rest cols)
   (declare (dynamic-extent cols))
-  (cond ((not ($matrixp m)) (merror "First argument to `addcol' must be a matrix"))
+  (cond ((not ($matrixp m)) (merror (intl:gettext "addcol: first argument must be a matrix; found ~M") m))
 	((null cols) m)
 	(t (let ((m ($transpose m)))
 	     (dolist (c cols ($transpose m))
 	       (setq m (addrow m ($transpose c))))))))
 
 (defun addrow (m r)
-  (cond ((not (mxorlistp r)) (merror "Illegal argument to `addrow' or `addcol'"))
+  (cond ((not (mxorlistp r)) (merror (intl:gettext "addrow or addcol: argument must be a matrix or list; found ~M") r))
 	((and (cdr m)
 	      (or (and (eq (caar r) 'mlist) (not (= (length (cadr m)) (length r))))
 		  (and (eq (caar r) '$matrix)
 		       (not (= (length (cadr m)) (length (cadr r))))
 		       (prog2 (setq r ($transpose r))
 			   (not (= (length (cadr m)) (length (cadr r))))))))
-	 (merror "Incompatible structure - `addrow'//`addcol'")))
+	 (merror (intl:gettext "addrow or addcol: incompatible structure."))))
   (append m (if (eq (caar r) '$matrix) (cdr r) (ncons r))))
 
 ;;;; ARRAYF
 
 (defmfun $arraymake (ary subs)
   (cond ((or (not ($listp subs)) (null (cdr subs)))
-	 (merror "Wrong type argument to `arraymake':~%~M" subs))
+	 (merror (intl:gettext "arraymake: second argument must be a list of one or more elements; found ~M") subs))
 	((eq (ml-typep ary) 'symbol)
 	 (cons (cons (getopr ary) '(array)) (cdr subs)))
 	(t (cons '(mqapply array) (cons ary (cdr subs))))))
@@ -693,7 +769,7 @@
 				(cons '(mlist) (mapcar #'1- dims)))))))
 	 (let ((gen (mgetl sym '(hashar array))) ary1)
 	   (cond ((null gen)
-		  (merror "Not an array - `arrayinfo':~%~M" ary))
+		  (merror (intl:gettext "arrayinfo: ~M is not an array.") ary))
 		 ((mfilep (cadr gen))
 		  (i-$unstore (ncons ary))
 		  (setq gen (mgetl ary '(hashar array)))))
@@ -745,11 +821,11 @@
 		aliaslist))))
 
 (defmspec $ordergreat (l)
-  (if greatorder (merror "Reordering is not allowed."))
+  (if greatorder (merror (intl:gettext "ordergreat: reordering is not allowed.")))
   (makorder (setq greatorder (reverse (cdr l))) '_))
 
 (defmspec $orderless (l)
-  (if lessorder (merror "Reordering is not allowed."))
+  (if lessorder (merror (intl:gettext "orderless: reordering is not allowed.")))
   (makorder (setq lessorder (cdr l)) '|#|))
 
 (defun makorder (l char)
@@ -773,9 +849,9 @@
 
 (defun $concat (&rest l)
   (when (null l)
-    (merror "concat: I need at least one argument."))
+    (merror (intl:gettext "concat: there must be at least one argument.")))
   (let ((result-is-a-string (or (numberp (car l)) (stringp (car l)))))
-    (setq l (mapcan #'(lambda (x) (unless (atom x) (merror "concat: argument is not an atom: ~M" x)) (string* x)) l))
+    (setq l (mapcan #'(lambda (x) (unless (atom x) (merror (intl:gettext "concat: argument must be an atom; found ~M") x)) (string* x)) l))
     (if result-is-a-string
       (coerce l 'string)
       (getalias (implode (cons '#\$ l))))))
