@@ -14,7 +14,14 @@
 
 (load-macsyma-macros rzmac)
 
-(declare-top (special var %p%i varlist plogabs half%pi nn* dn*))
+(declare-top (special var %p%i varlist plogabs half%pi nn* dn* $factlim
+                      $beta_expand))
+
+(defmvar $gammalim 10000
+  "Controls simplification of gamma for rational number arguments.")
+
+(defvar $gamma_expand nil
+  "Expand gamma(z+n) for n an integer when T.") 
 
 (defmfun simpplog (x vestigial z)
   (declare (ignore vestigial))
@@ -22,7 +29,7 @@
      (oneargcheck x)
      (setq check x)
      (setq x (simpcheck (cadr x) z))
-     (cond ((equal 0 x) (merror "plog(0) is undefined"))
+     (cond ((equal 0 x) (merror (intl:gettext "plog: plog(0) is undefined.")))
 	   ((among var x)	;This is used in DEFINT. 1/19/81. -JIM
 	    (return (eqtest (list '(%plog) x) check))))
      (newvar x)
@@ -46,11 +53,11 @@
 	      ((eq y '$neg)
 	       (return (add2 %p%i
 			     (simpln (list '(%log) (list '(mtimes) -1 x)) 1 nil))))
-	      (t (merror "plog(0) is undefined"))))
+	      (t (merror (intl:gettext "plog: plog(0) is undefined.")))))
        ((and (equal ($imagpart (setq z (div* x '$%i))) 0)
 	     (setq y ($asksign z)))
 	(cond
-	  ((equal y '$zero) (merror "plog(0) is undefined"))
+	  ((equal y '$zero) (merror (intl:gettext "plog(0) is undefined.")))
 	  (t (cond ((eq y '$pos) (setq y 1))
 		   ((eq y '$neg) (setq y -1)))
 	     (return (add2* (simpln (list '(%log)
@@ -68,13 +75,13 @@
 	      (cond ((equal i '$pos) (return (simplify half%pi)))
 		    ((equal i '$neg)
 		     (return (mul2 -1 (simplify half%pi))))
-		    (t (merror "atan(0//0) has been generated."))))
+		    (t (merror (intl:gettext "plog: encountered atan(0/0).")))))
 	     ((zerop1 i)
 	      (cond ((floatp r) (setq $numer t)))
 	      (setq r ($asksign r))
 	      (cond ((equal r '$pos) (return 0))
 		    ((equal r '$neg) (return (simplify '$%pi)))
-		    (t (merror "atan(0//0) has been generated."))))
+		    (t (merror (intl:gettext "plog: encountered atan(0/0).")))))
 	     ((and (among '%cos r) (among '%sin i))
 	      (setq var 'xz)
 	      (numden (div* r i))
@@ -101,7 +108,10 @@
 		     ((alike1 r '((mexpt) 3 ((rat) -1 2)))
 		      (archk a b (list '(mtimes) '((rat) 1 6) '$%pi))))))))
 
-(defmfun simpbinocoef (x vestigial z) 
+;; Binomial has Mirror symmetry
+(defprop %binomial t commutes-with-conjugate)
+
+(defun simpbinocoef (x vestigial z)
   (declare (ignore vestigial))
   (twoargcheck x)
   (let ((u (simpcheck (cadr x) z))
@@ -109,13 +119,32 @@
 	(y))
     (cond ((integerp v)
 	   (cond ((minusp v)
-		  (if (and (integerp u) (minusp u) (< v u)) (bincomp u (- u v)) 0))
+		  (if (and (integerp u) (minusp u) (< v u))
+		      (bincomp u (- u v))
+		      0))
 		 ((or (zerop v) (equal u v)) 1)
-		 ((and (integerp u) (not (minusp u))) (bincomp u (min v (- u v))))
+		 ((and (integerp u) (not (minusp u)))
+		  (bincomp u (min v (- u v))))
 		 (t (bincomp u v))))
-	  ((integerp (setq y (sub u v))) (bincomp u y))
-	  ((and (floatp u) (floatp v)) ($makegamma (list '(%binomial) u v)))
-	  (t (eqtest (list '(%binomial) u v) x)))))
+          ((integerp (setq y (sub u v)))
+           (cond ((zerop1 y)
+                  ;; u and v are equal, simplify not if argument can be negative
+                  (if (member ($csign u) '($pnz $pn $neg $nz))
+                      (eqtest (list '(%binomial) u v) x)
+                      (bincomp u y)))
+                 (t (bincomp u y))))
+          ((complex-float-numerical-eval-p u v)
+           ;; Numercial evaluation for real and complex floating point numbers.
+           (let (($numer t) ($float t))
+             ($rectform
+               ($float 
+                 ($makegamma (list '(%binomial) ($float u) ($float v)))))))
+          ((complex-bigfloat-numerical-eval-p u v)
+           ;; Numerical evaluation for real and complex bigfloat numbers.
+           ($rectform
+             ($bfloat
+               ($makegamma (list '(%binomial) ($bfloat u) ($bfloat v))))))
+          (t (eqtest (list '(%binomial) u v) x)))))
 
 (defun bincomp (u v) 
   (cond ((minusp v) 0)
@@ -140,28 +169,126 @@
 
 (defmvar $beta_args_sum_to_integer nil)
 
+;;; The Beta funtion has mirror symmetry
+(defprop $beta t commutes-with-conjugate)
+
 (defmfun simpbeta (x vestigial z &aux check)
   (declare (ignore vestigial))
   (twoargcheck x)
   (setq check x)
   (let ((u (simpcheck (cadr x) z)) (v (simpcheck (caddr x) z)))
     (cond ((or (zerop1 u) (zerop1 v))
-	   (if errorsw (throw 'errorsw t) (merror "Zero argument to `beta'")))
-	  ((or (and (floatp u) (floatp v))
-	       (and $numer (numberp u) (numberp v)))
-	   ($makegamma (list '($beta) u v)))
-	  ((or (and (integerp u) (plusp u)) (and (integerp v) (plusp v)))
-	   (setq x (add2 u v))
-	   (power (mul2 (sub x 1)
-			(simplifya (list '(%binomial)
-					 (sub x 2)
-					 (sub (if (and (integerp u) (plusp u)) u v) 1))
-				   t))
-		  -1))
-	  ((and (integerp u) (integerp v))
-	   (mul2* (div* (list '(mfactorial) (1- u))
-			(list '(mfactorial) (+ u v -1)))
-		  (list '(mfactorial) (1- v))))
+           (if errorsw 
+               (throw 'errorsw t) 
+               (merror 
+                 (intl:gettext "beta: expected nonzero arguments; found ~M, ~M")
+                               u v)))
+
+          ;; Check for numerical evaluation in float precision
+      	  ((complex-float-numerical-eval-p u v)
+           (cond
+             ;; We use gamma(u)*gamma(v)/gamma(u+v) for numerical evaluation.
+             ;; Therefore u, v or u+v can not be a negative integer or a
+             ;; floating point representation of a negative integer.
+             ((and (or (not (numberp u))
+                       (> u 0)
+                       (not (= (nth-value 1 (truncate u)) 0)))
+              (and (or (not (numberp v))
+                       (> v 0)
+                       (not (= (nth-value 1 (truncate v)) 0)))
+              (and (or (not (numberp (add u v)))
+                       (> (add v u) 0)
+                       (not (= (nth-value 1 ($truncate (add u v))) 0))))))
+	      ($rectform 
+	        (power ($float '$%e)
+	               (add ($log_gamma ($float u))
+                            ($log_gamma ($float v))
+                            (mul -1 ($log_gamma ($float (add u v))))))))
+             ((or (and (numberp u)
+                       (> u 0)
+                       (= (nth-value 1 (truncate u)) 0)
+                       (not (and (mnump v)
+                                 (eq ($sign (sub ($truncate v) v)) '$zero)
+                                 (eq ($sign v) '$neg)
+                                 (eq ($sign (add u v)) '$pos)))
+                       (setq u (truncate u)))
+                  (and (numberp v)
+                       (> v 0)
+                       (= (nth-value 1 (truncate u)) 0)
+                       (not (and (mnump u)
+                                 (eq ($sign (sub ($truncate u) u)) '$zero)
+                                 (eq ($sign u) '$neg)
+                                 (eq ($sign (add u v)) '$pos)))
+                       (setq v (truncate v))))
+              ;; One value is representing a negative integer, the other a
+              ;; positive integer and the sum is negative. Expand.
+              ($rectform ($float (beta-expand-integer u v))))
+             (t
+               (eqtest (list '($beta) u v) check))))
+
+          ;; Check for numerical evaluation in bigfloat precision
+          ((complex-bigfloat-numerical-eval-p u v)
+           (let (($ratprint nil))
+             (cond
+               ((and (or (not (mnump u))
+                         (eq ($sign u) '$pos)
+                         (not (eq ($sign (sub ($truncate u) u)) '$zero)))
+                     (or (not (mnump v))
+                         (eq ($sign v) '$pos)
+                         (not (eq ($sign (sub ($truncate v) v)) '$zero)))
+                     (or (not (mnump (add u v)))
+                         (eq ($sign (add u v)) '$pos)
+                         (not (eq ($sign (sub ($truncate (add u v))
+                                              (add u v)))
+                                  '$zero))))
+                ($rectform 
+                  (power ($bfloat'$%e)
+                         (add ($log_gamma ($bfloat u))
+                              ($log_gamma ($bfloat v))
+                              (mul -1 ($log_gamma ($bfloat (add u v))))))))
+               ((or (and (mnump u)
+                         (eq ($sign u) '$pos)
+                         (eq ($sign (sub ($truncate u) u)) '$zero)
+                         (not (and (mnump v)
+                              (eq ($sign (sub ($truncate v) v)) '$zero)
+                              (eq ($sign v) '$neg)
+                              (eq ($sign (add u v)) '$pos)))
+                         (setq u ($truncate u)))
+                    (and (mnump v)
+                         (eq ($sign v) '$pos)
+                         (eq ($sign (sub ($truncate v) v)) '$zero)
+                         (not (and (mnump u)
+                              (eq ($sign (sub ($truncate u) u)) '$zero)
+                              (eq ($sign u) '$neg)
+                              (eq ($sign (add u v)) '$pos)))
+                         (setq v ($truncate v))))
+                ($rectform ($bfloat (beta-expand-integer u v))))
+               (t
+                 (eqtest (list '($beta) u v) check)))))
+
+      	  ((or (and (and (integerp u)
+	                 (plusp u))
+	            (not (and (mnump v)
+	                      (eq ($sign (sub ($truncate v) v)) '$zero)
+      	                      (eq ($sign v) '$neg)
+	                      (eq ($sign (add u v)) '$pos))))
+	       (and (and (integerp v) 
+	                 (plusp v))
+                    (not (and (mnump u)
+                              (eq ($sign (sub ($truncate u) u)) '$zero)
+                              (eq ($sign u) '$neg)
+                              (eq ($sign (add u v)) '$pos)))))
+           ;; Expand for a positive integer. But not if the other argument is 
+           ;; a negative integer and the sum of the integers is not negative.
+           (beta-expand-integer u v))
+
+;;; At this point both integers are negative. This code does not work for
+;;; negative integers. The factorial function is not defined.
+;	  ((and (integerp u) (integerp v))
+;	   (mul2* (div* (list '(mfactorial) (1- u))
+;			(list '(mfactorial) (+ u v -1)))
+;		  (list '(mfactorial) (1- v))))
+
 	  ((or (and (ratnump u) (ratnump v) (integerp (setq x (addk u v))))
 	       (and $beta_args_sum_to_integer
 		    (integerp (setq x (expand1 (add2 u v) 1 1)))))
@@ -171,32 +298,114 @@
 				(add2 (1- x) (neg w))
 				(1- x)))
 		   `((%sin) ((mtimes) ,w $%pi)))))
+          
+          ((and $beta_expand (mplusp u) (integerp (cadr u)))
+           ;; Expand beta(a+n,b) where n is an integer.
+           (let ((n (cadr u))
+                 (u (simplify (cons '(mplus) (cddr u)))))
+             (beta-expand-add-integer n u v)))
+          
+          ((and $beta_expand (mplusp v) (integerp (cadr v)))
+           ;; Expand beta(a,b+n) where n is an integer.
+           (let ((n (cadr v))
+                 (v (simplify (cons '(mplus) (cddr v)))))
+             (beta-expand-add-integer n v u)))
+                  
 	  (t (eqtest (list '($beta) u v) check)))))
 
+(defun beta-expand-integer (u v)
+  ;; One of the arguments is a positive integer. Do an expansion.
+  ;; BUT for a negative integer as second argument the expansion is only
+  ;; possible when the sum of the integers is negative too.
+  ;; This routine expects that the calling routine has checked this.
+  (let ((x (add u v)))
+    (power
+      (mul (sub x 1)
+           (simplify
+             (list '(%binomial)
+                   (sub x 2)
+                   (sub (if (and (integerp u) (plusp u)) u v) 1))))
+      -1)))
+
+(defun beta-expand-add-integer (n u v)
+  (if (plusp n)
+      (mul (simplify (list '($pochhammer) u n))
+           (power (simplify (list '($pochhammer) (add u v) n)) -1)
+           (simplify (list '($beta) u v)))
+      (mul (simplify (list '($pochhammer) (add u v n) (- n)))
+           (power (simplify (list '($pochhammer) (add u n) (- n))) -1)
+           (simplify (list '($beta) u v)))))
+
 (defmfun simpgamma (x vestigial z)
   (declare (ignore vestigial))
   (oneargcheck x)
-  (let* ((j (simpcheck (cadr x) z))
-	 (jr ($realpart j))
-	 (ji ($imagpart j)))
-    (cond ((floatp j) (gammafloat j))
-	  (($bfloatp j) (mfuncall '$bffac (m+ j -1) $fpprec))
-	  ((and (numberp jr) 
-		(numberp ji)
-		(or $numer (floatp jr) (floatp ji)))
-	   (complexify (gamma-lanczos (complex (float jr)
-					       (float ji)))))
-	  ((or (not (mnump j))
-	       (ratgreaterp (simpabs (list '(%abs) j) 1 t) $gammalim))
-	   (eqtest (list '(%gamma) j) x))
+  (let ((j (simpcheck (cadr x) z)))
+    (cond ((and (floatp j)
+                (or (zerop j)
+                    (and (< j 0)
+                         (zerop (nth-value 1 (truncate j))))))
+           (merror (intl:gettext "gamma: gamma(~:M) is undefined.") j))
+          ((float-numerical-eval-p j) (gammafloat ($float j)))
+          ((and ($bfloatp j)
+                (or (zerop1 j)
+                    (and (eq ($sign j) '$neg)
+                         (zerop1 (sub j ($truncate j))))))
+           (merror (intl:gettext "gamma: gamma(~:M) is undefined.") j))
+          ((bigfloat-numerical-eval-p j) 
+           ;; Adding 4 digits in the call to bffac. For $fpprec up to about 256
+           ;; and an argument up to about 500.0 the accuracy of the result is
+           ;; better than 10^(-$fpprec).
+	   (let ((result (mfuncall '$bffac (m+ ($bfloat j) -1) (+ $fpprec 4))))
+	     ;; bigfloatp will round the result to the correct fpprec
+	     (bigfloatp result)))
+	  ((complex-float-numerical-eval-p j)
+           (complexify (gamma-lanczos (complex ($float ($realpart j))
+                                               ($float ($imagpart j))))))
+          ((complex-bigfloat-numerical-eval-p j)
+           ;; Adding 4 digits in the call to cbffac. See comment above.
+	   (let ((result
+		  (mfuncall '$cbffac 
+			    (add -1 ($bfloat ($realpart j)) 
+				 (mul '$%i ($bfloat ($imagpart j))))
+			    (+ $fpprec 4))))
+	     (add (bigfloatp ($realpart result))
+		  (mul '$%i (bigfloatp ($imagpart result))))))
+          ((taylorize (mop x) (cadr x)))
+          ((eq j '$inf) '$inf) ; Simplify to $inf to be more consistent.
+          ((and $gamma_expand
+                (mplusp j) 
+                (integerp (cadr j)))
+           ;; Expand gamma(z+n) for n an integer.
+           (let ((n (cadr j))
+                 (z (simplify (cons '(mplus) (cddr j)))))
+             (cond 
+               ((> n 0)
+                (mul (simplify (list '($pochhammer) z n))
+                     (simplify (list '(%gamma) z))))
+               ((< n 0)
+                (setq n (- n))
+                (div (mul (power -1 n) (simplify (list '(%gamma) z)))
+                     ;; We factor to get the order (z-1)*(z-2)*...
+                     ;; and not (1-z)*(2-z)*... 
+                     ($factor
+                       (simplify (list '($pochhammer) (sub 1 z) n))))))))
 	  ((integerp j)
-	   (cond ((> j 0) (simpfact (list '(mfactorial) (1- j)) 1 nil))
+	   (cond ((> j 0)
+                  (cond ((<= j $factlim)
+                         ;; Positive integer less than $factlim. Evaluate.
+                         (simplify (list '(mfactorial) (1- j))))
+                         ;; Positive integer greater $factlim. Noun form.
+                        (t (eqtest (list '(%gamma) j) x))))
+                 ;; Negative integer. Throw a Maxima error.
 		 (errorsw (throw 'errorsw t))
-		 (t (merror "gamma(~:M) is undefined" j))))
-	  ($numer (gammafloat (fpcofrat j)))
+		 (t (merror (intl:gettext "gamma: gamma(~:M) is undefined.") j))))
 	  ((alike1 j '((rat) 1 2))
 	   (list '(mexpt simp) '$%pi j))
-	  ((or (ratgreaterp j 1) (ratgreaterp 0 j)) (gammared j))
+          ((and (mnump j)
+                (ratgreaterp $gammalim (simplify (list '(mabs) j)))
+                (or (ratgreaterp j 1) (ratgreaterp 0 j)))
+           ;; Expand for rational numbers less than $gammalim.
+           (gammared j))
 	  (t (eqtest (list '(%gamma) j) x)))))
 
 (defun gamma (y) ;;; numerical evaluation for 0 < y < 1
@@ -247,8 +456,8 @@
 ;;
 ;; to handle the case of Re(z) <= 0.
 ;;
-;; See http://winnie.fit.edu/~gabdo/gamma.m for some matlab code to
-;; compute this and http://winnie.fit.edu/~gabdo/gamma.txt for a nice
+;; See http://my.fit.edu/~gabdo/gamma.m for some matlab code to
+;; compute this and http://my.fit.edu/~gabdo/gamma.txt for a nice
 ;; discussion of Lanczos method and an improvement of Lanczos method.
 ;;
 ;;
@@ -285,47 +494,84 @@
 	;; Gamma(z) = pi/z/sin(pi*z)/Gamma(-z)
 	;;
 	;; If z is a negative integer, Gamma(z) is infinity.  Should
-	;; we test for this?  Throw an error?  What
+	;; we test for this?  Throw an error?
+        ;; The test must be done by the calling routine.
 	(/ (float pi)
 	   (* (- z) (sin (* (float pi) z))
 	      (gamma-lanczos (- z))))
 	(let* ((z (- z 1))
 	       (zh (+ z 1/2))
 	       (zgh (+ zh 607/128))
-	       (zp (expt zgh (/ zh 2)))
 	       (ss 
 		(do ((sum 0.0)
 		     (pp (1- (length c)) (1- pp)))
 		    ((< pp 1)
 		     sum)
 		  (incf sum (/ (aref c pp) (+ z pp))))))
-	  (* (sqrt (float (* 2 pi)))
-	     (+ ss (aref c 0))
-	     (* zp (exp (- zgh)) zp))))))
+          (let ((result 
+                 ;; We check for an overflow. The last positive value in 
+                 ;; double-float precicsion for which Maxima can calculate 
+                 ;; gamma is ~171.6243 (CLISP 2.46 and GCL 2.6.8)
+                 (ignore-errors
+		   (let ((zp (expt zgh (/ zh 2))))
+		     (* (sqrt (float (* 2 pi)))
+			(+ ss (aref c 0))
+			(* (/ zp (exp zgh)) zp))))))
+            (cond ((null result)
+                   ;; No result. Overflow.
+                   (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
+                  ((or (float-nan-p (realpart result))
+                       (float-inf-p (realpart result)))
+                   ;; Result, but beyond extreme values. Overflow.
+                   (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
+                  (t result)))))))
 
 (defun gammafloat (a)
-  (realpart (gamma-lanczos (complex a 0.0))))
+  (let ((a (float a)))
+    (cond ((minusp a)
+	   (/ (float (- pi))
+	      (* a (sin (* (float pi) a)))
+	      (gammafloat (- a))))
+	  ((< a 10)
+	   (slatec::dgamma a))
+	  (t
+	   (let ((result
+		  (let ((c (* (sqrt (* 2 (float pi)))
+			      (exp (slatec::d9lgmc a)))))
+		    (let ((v (expt a (- (* .5d0 a) 0.25d0))))
+		      (* v
+			 (/ v (exp a))
+			 c)))))
+	     (if (or (float-nan-p result)
+		     (float-inf-p result))
+		 (merror (intl:gettext "gamma: overflow in GAMMAFLOAT."))
+		 result))))))
+
 
 (declare-top (special $numer $trigsign))
 
-(defmfun simperf (x vestigial z &aux y)
-  (declare (ignore vestigial))
-  (oneargcheck x)
-  (setq y (simpcheck (cadr x) z))
-  (cond ((zerop1 y) y)
-	((or (floatp y) (and $numer (integerp y))) (erf (float y)))
-	((eq y '$inf) 1)
-	((eq y '$minf) -1)
-	;;((and $trigsign (mminusp* y)) (neg (list '(%erf simp) (neg y))))
-	((and $trigsign (great (neg y) y)) (neg (take '(%erf) (neg y))))
-	(t (eqtest (list '(%erf) y) x))))
+;;; The Error functions Erf, Erfc, Erfi and the function Generalized Erf have 
+;;; been implemented as simplifying functions. The code is moved to the file
+;;; gamma.lisp and commented out at this place. 10/2008 DK.
+
+;(defmfun simperf (x vestigial z &aux y)
+;  (declare (ignore vestigial))
+;  (oneargcheck x)
+;  (setq y (simpcheck (cadr x) z))
+;  (cond ((zerop1 y) y)
+;	((or (floatp y) (and $numer (integerp y))) (erf (float y)))
+;	((eq y '$inf) 1)
+;	((eq y '$minf) -1)
+;	;;((and $trigsign (mminusp* y)) (neg (list '(%erf simp) (neg y))))
+;	((and $trigsign (great (neg y) y)) (neg (take '(%erf) (neg y))))
+;	(t (eqtest (list '(%erf) y) x))))
 
 
-(defmfun erf (y)
-  (slatec:derf (float y)))
+;(defmfun erf (y)
+;  (slatec:derf (float y)))
 
-(defmfun erfc (y)
-  (slatec:derfc (float y)))
+;(defmfun erfc (y)
+;  (slatec:derfc (float y)))
 
 (defmfun $zeromatrix (m n) ($ematrix m n 0 1 1))
 
@@ -337,7 +583,7 @@
 	   ((not (and (fixnump m) (fixnump n)
 		      (fixnump i) (fixnump j)
 		      (> m 0) (> n 0) (> i 0) (> j 0)))
-	    (merror "Incorrect argument to `ematrix':~%~M"
+	    (merror (intl:gettext "ematrix: arguments must be positive integers; found ~M")
 		    (list '(mlist simp) m n i j) )))
      loop (cond ((= m i) (setq row (onen j n var 0)) (go on))
 		((zerop m) (return (cons '($matrix) (mxc ans)))))
@@ -363,7 +609,7 @@
      (if (not ($listp varl)) (improper-arg-err varl '$coefmatrix))
      (dolist (v (cdr varl))
        (if (and (not (atom v)) (member (caar v) '(mplus mtimes) :test #'eq))
-	   (merror "Improper variable to `coefmatrix':~%~M" v)))
+	   (merror (intl:gettext "coefmatrix: variables cannot be '+' or '*' expressions; found ~M") v)))
      (setq eql (nreverse (mapcar #'meqhk (cdr eql)))
 	   varl (reverse (cdr varl)))
      loop1(if (null eql) (return (cons '($matrix) (mxc ans))))
@@ -383,7 +629,7 @@
   (prog (row column vector matrix sym symvector)
      (cond ((or (not (fixnump rows))
 		(not (fixnump columns)))
-	    (merror "`entermatrix' called with non-integer arguments")))
+	    (merror (intl:gettext "entermatrix: arguments must be integers; found ~M, ~M") rows columns)))
      (setq row 0)
      (unless (= rows columns) (setq sym nil) (go oloop))
      quest (format t "~%Is the matrix  1. Diagonal  2. Symmetric  3. Antisymmetric  4. General~%")
@@ -523,7 +769,7 @@
        next (decf ix)
        (go loop)
        out
-       (cond ($dispflag (mtell "Solution~%")))
+       (cond ($dispflag (mtell (intl:gettext "Solution:~%"))))
        (setq sol (list '(mlist)) chk (checklabel $linechar))
        (do ((ll zz (cdr ll)))
 	   ((null ll))
