@@ -25,23 +25,23 @@
 (setf (get '$set  'dimension) 'dimension-match)
 
 ;; Parse {a, b, c} into set(a, b, c).
-;; Don't bother with DEF-NUD etc -- matchfix + ::= works just fine.
-;; Well, MDEFMACRO is a little too zealous in this context, since we
-;; don't really want this macro defn to show up on the $MACROS infolist.
-;; (Maybe it would be cleanest to append built-in defns to FOO::$MACROS
-;; where FOO is something other than MAXIMA, but that awaits
-;; regularization of package use within Maxima.)
 
-(eval-when
-    #+gcl (load eval)
-    #-gcl (:load-toplevel :execute)
-    ;; matchfix ("{", "}")
-    (meval '(($matchfix) "{" "}"))
-    ;; "{" ([L]) ::= buildq ([L], set (splice (L)));
-    (let ((new-defn
-	   (meval '((mdefmacro) ((${) ((mlist) $l)) (($buildq) ((mlist) $l) (($set) (($splice) $l)))))))
-      ;; Simpler to patch up $MACROS here, than to replicate the functionality of MDEFMACRO.
-      (setq $macros (delete (cadr new-defn) $macros :test #'equal))))
+(putopr "{" '$set)
+
+(def-nud-equiv |$}| delim-err)
+(def-led-equiv |$}| erb-err)
+(def-lbp     |$}| 5.)
+
+(def-nud-equiv	|${| parse-matchfix)
+(def-match	|${| |$}|)
+(def-lbp	|${| 200.)
+;No RBP
+(def-mheader	|${| ($set))
+(def-pos	|${| $any)
+;No LPOS
+;No RPOS
+
+(def-operator "{" '$any nil '$any nil nil nil nil '(nud . parse-matchfix) 'msize-matchfix 'dimension-match "}")
 
 ;; Support for TeXing sets. If your mactex doesn't TeX the empty set
 ;; correctly, get the latest mactex.lisp.
@@ -50,10 +50,7 @@
 (defprop $set (("\\left \\{" ) " \\right \\}") texsym)
 
 (defun require-set (x context-string)
-  (cond (($setp x) 
-	 (cdr (simp-set x 1 nil)))
-	(t 
-	 (merror "Function ~:M expects a set, instead found ~:M" context-string x))))
+  (if ($setp x) (cdr x) (merror (intl:gettext "~:M: argument must be a set; found: ~:M") context-string x)))
 
 ;; If x is a Maxima list, return a Lisp list of its members; otherwise,
 ;; signal an error. Unlike require-set, the function require-list does not
@@ -61,7 +58,7 @@
 
 (defun require-list (x context-string)
   (if ($listp x) (cdr x)
-    (merror "Function ~:M expects a list, instead found ~:M" context-string x)))
+    (merror (intl:gettext "~:M: argument must be a list; found: ~:M") context-string x)))
 
 ;; If x is a Maxima list or set, return a Lisp list of its members; otherwise,
 ;; signal an error.  Unlike require-set, the function require-list-or-set 
@@ -69,7 +66,7 @@
 
 (defun require-list-or-set (x context-string)
   (if (or ($listp x) ($setp x)) (cdr x)
-    (merror "Function ~:M expects a list or a set, instead found ~:M" context-string x)))
+    (merror (intl:gettext "~:M: argument must be a list or a set; found: ~:M") context-string x)))
 
 ;; When a is a list, return a list of the unique elements of a.
 ;; Otherwise just return a.
@@ -102,9 +99,10 @@
 ;; full_listify(a) converts all sets in a into lists.
 
 (defun $full_listify (a)
-  (cond ((atom a) a)
-	(($setp a) `((mlist simp) ,@(mapcar #'$full_listify (cdr a))))
-	(t `(,(car a) ,@(mapcar #'$full_listify (cdr a))))))
+  (setq a ($ratdisrep a))
+  (cond (($mapatom a) a)
+	(($setp a) (simplify (cons (list 'mlist) (mapcar #'$full_listify (cdr a)))))
+	(t (simplify (cons (car a) (mapcar #'$full_listify (cdr a)))))))
 
 (defprop $set simp-set operators)
 
@@ -126,14 +124,11 @@
 (defun $setp (a)
   (and (consp a) (consp (car a)) (eq (caar a) '$set)))
 
-;; Return the cardinality of a set; if the argument is a list, convert it to a
-;; set. Works even when simp : false.  For example,
-
-;; (C1) cardinality(set(a,a,a)), simp : false;
-;; (D1) 				   1
+;; Return the cardinality of a set. This function works even when $simp is false.
  
 (defun $cardinality (a)
-  (length (require-set a "$cardinality")))
+  (if $simp (length (require-set a "$cardinality"))
+    (let (($simp t)) ($cardinality (simplify a)))))
 
 ;; Return true iff a is a subset of b. If either argument is a list, first 
 ;; convert it to a set. Signal an error if a or b aren't lists or sets.
@@ -250,7 +245,7 @@
       (setq b (mfuncall f x))
       (cond ((eq t b) (push x acc))
 	    ((not (or (eq b nil) (eq b '$unknown)))
-	     (merror "The function ~:M doesn't evaluate to a boolean for ~:M" f x))))))
+	     (merror (intl:gettext "subset: ~:M(~:M) evaluates to a non-boolean.") f x))))))
 	     
 ;; Return a list of three sets. The first set is the subset of a for which
 ;; the predicate f evaluates to true, the second is the subset of a
@@ -267,26 +262,27 @@
       (cond ((eq t b) (push x t-acc))
 	    ((or (eq b nil) (eq b '$unknown)) (push x f-acc))
 	    (t 
-	     (merror "The function ~:M doesn't evaluate to a boolean for ~:M" f x))))))
+	     (merror (intl:gettext "partition_set: ~:M(~:M) evaluates to a non-boolean.") f x))))))
 	       
-;; Let a = (a1,a2,...,an), where each ai is either a list or a set.
-;; Return {x | x is a member of exactly one ai}.  When n = 2, this
-;; is the standard symmdifference of sets; thus symmdifference(a,b)
-;; evaluates to the union of a - b and b - a. Further symmdifference() -> set()
-;; and symmdifference(a) -> a.  Signal an error when one or more of the
-;; ai are not lists or sets.
+;; The symmetric difference of sets, that is (A-B) union (B - A), is associative.
+;; Thus the symmetric difference extends unambiguously to n-arguments.
 
-;; After completing dotimes, the list acc is non-redundant -- so 
-;; we only need to sort acc.
+(defun $symmdifference (&rest l)
+  (let ((acc nil))
+    (dolist (lk l (cons '($set simp) acc))
+      (setq acc (set-symmetric-difference acc (require-set lk "$symmdifference"))))))
+            
+;; Return {x | x in exactly one set l1, l2, ...}
 
-(defun $symmdifference (&rest a)
-  (let ((acc) (n (length a)))
-    (declare (fixnum n))
-    (setq a (mapcar #'(lambda (x) (require-set x "$symmdifference")) a))
-    (dotimes (i n)
-      (setq acc (append acc (reduce #'sset-difference a)))
-      (setq a (cons (car (last a)) (butlast a))))
-    `(($set simp) ,@(sort acc '$orderlessp))))
+(defun $in_exactly_one (&rest l)
+  ;; u = union of l1, l2,...
+  ;; r = members that are in two or more l1, l2, ...
+  (let ((u nil) (r nil))
+    (dolist (lk l)
+      (setq lk (require-set lk "$in_exactly_one"))
+      (setq r (set-union r (set-intersect u lk)))
+      (setq u (set-union u lk)))
+    (cons '($set simp) (sset-difference u r))))
 
 ;; When k is a positive integer, return the set of all subsets of a 
 ;; that have exactly k elements; when k is nil, return the power set
@@ -600,7 +596,7 @@
 	       ((eq s '$max)
 		(setq s 1))
 	       (t
-		(merror "The third argument of extremal_subset must be max or min; instead found ~:M" s)))
+		(merror (intl:gettext "extremal_subset: third argument must be 'max or 'min; found: ~:M") s)))
 	 (let* ((max-subset (nth 0 a))
 		(mx (mul s (mfuncall f max-subset)))
 		(x))
@@ -623,7 +619,7 @@
   (let (($prederror nil) (b))
     (setq b (mevalp (mfuncall f x y)))
     (if (or (eq b t) (eq b nil)) b
-      (merror  "~:M(~:M,~:M) doesn't evaluate to a boolean" f x y))))
+      (merror (intl:gettext "equiv_classes: ~:M(~:M, ~:M) evaluates to a non-boolean.") f x y))))
   
     
 ;; Return the set of equivalence classes of f on the set l.  The
@@ -639,7 +635,7 @@
        (acc)
        (tail)
        (x))
-      ((null l) (cons '($set) (mapcar #'(lambda (x) (cons '($set) x)) acc)))
+      ((null l) (simplify (cons '($set) (mapcar #'(lambda (x) (cons '($set) x)) acc))))
     (setq x (car l))
     (setq tail (member-if #'(lambda (z) (bool-checked-mfuncall f x (car z))) acc))
     (cond ((null tail)
@@ -695,8 +691,7 @@
 	     (incf k))
 	   `(($set) ,@acc)))
 	(t
-	 (merror "The optional second argument to set_partitions must be
-a positive integer; instead found ~:M" n-sub))))
+	 (merror (intl:gettext "set_partitions: second argument must be a positive integer; found: ~:M") n-sub))))
 
 (defun set-partitions (a n)
   (cond ((= n 0)
@@ -715,7 +710,7 @@ a positive integer; instead found ~:M" n-sub))))
 	     (setq s (cdr pj))
 	     (while (not (null s))
 	       (setq z (pop s))
-	       (setq acc (cons `(($set) ,@w ,($adjoin x z) ,@s) acc))
+	       (setq acc (cons (simplifya `(($set) ,@w ,($adjoin x z) ,@s) t) acc))
 	       (setq w (cons z w))))
 	     	   
 	   (setq x `(($set) ,x))
@@ -1076,7 +1071,7 @@ a positive integer; instead found ~:M" n-sub))))
 	 (reduce #'(lambda (x y) (mfuncall f x y)) s :from-end left 
 		 :initial-value init))
 	((null s)
-	 (merror "Undefined - either give initial value or a nonempty set"))
+	 (merror (intl:gettext "~a: either a nonempty set or initial value must be given.") fn))
 	(t
 	 (reduce #'(lambda (x y) (mfuncall f x y)) s :from-end left))))
 
@@ -1089,11 +1084,11 @@ a positive integer; instead found ~:M" n-sub))))
 
 (defun xappend (s)
   #+(or cmu scl)
-  (cons '(mlist) (apply 'append (mapcar #'(lambda (x) 
-					    (require-list x "$append")) s)))
+  (cons '(mlist) (apply 'append (mapcar #'(lambda (x)
+                        (require-list x "$append")) s)))
   #-(or cmu scl)
   (let ((acc))
-    (dolist (si s (cons '(mlist) acc))
+    (dolist (si (reverse s) (cons '(mlist) acc))
       (setq acc (append (require-list si "$append") acc)))))
 
 (def-nary 'mand (s) (mevalp (cons '(mand) s)) t)
@@ -1110,16 +1105,28 @@ a positive integer; instead found ~:M" n-sub))))
 ;; When there isn't a Maxima function we can call (actually when (get op '$nary) 
 ;; returns nil) we give up and use rl-reduce with left-associativity.
 
+
 (defun $xreduce (f s &optional (init 'no-init))
-  (let ((op (if (atom f) ($verbify f) nil)))
-    (cond ((get op '$nary)
+  (let* ((op-props (get (if (atom f) ($verbify f) nil) '$nary))
+	 (opfn  (if (consp op-props) (car op-props) nil)))
+  
+    (cond (opfn
 	   (setq s (require-list-or-set s "$xreduce"))
-	   (if (not (equal init 'no-init)) (setq s (cons init s)))
-					;(print "...using nary function")
-	   (mapply1 op s f nil))
+	   (if (not (equal init 'no-init))
+	       (setq s (cons init s)))
+	  
+	   (if (null s)
+	       (cadr op-props)        ; is this clause really needed?
+	     
+	     (funcall opfn s)))
+
+	  (op-props
+	   ($apply f ($listify s)))
+	  
 	  (t
 	   (rl-reduce f ($listify s) nil init "$xreduce")))))
 
+
 ;; Extend a function f : S x S -> S to n arguments using a minimum depth tree.
 ;; The function f should be nary (associative); otherwise, the result is somewhat 
 ;; difficult to describe -- for an odd number of arguments, we favor the left side of the tree.
@@ -1128,7 +1135,7 @@ a positive integer; instead found ~:M" n-sub))))
   (setq a (require-list-or-set a "$tree_reduce"))
   (if (not (equal init 'no-init)) (push init a))
   (if (null a)
-      (merror "When the argument is empty, the optional argument must be defined"))
+      (merror (intl:gettext "tree_reduce: either a nonempty set or initial value must be given.")))
   
   (let ((acc) (x) (doit nil))
     (while (consp a)
@@ -1219,6 +1226,8 @@ a positive integer; instead found ~:M" n-sub))))
   (cond ((or (eq x t) (eq x nil) (not $prederror)) x)
 	((eq x '$unknown) nil)
 	(t
+         ;; FOLLOWING MESSAGE IS UNREACHABLE FROM WHAT I CAN TELL
+         ;; SINCE MAYBE RETURNS T, NIL, OR '$UNKNOWN
 	 (merror "Predicate isn't true/false valued; maybe you want to set 'prederror' to false"))))
     
 (defun checked-or (x)
@@ -1226,6 +1235,8 @@ a positive integer; instead found ~:M" n-sub))))
   (cond ((or (eq x t) (eq x nil) (not $prederror)) x)
 	((eq x '$unknown) nil)
 	(t
+         ;; FOLLOWING MESSAGE IS UNREACHABLE FROM WHAT I CAN TELL
+         ;; SINCE MAYBE RETURNS T, NIL, OR '$UNKNOWN
 	 (merror "Predicate isn't true/false valued; maybe you want to set 'prederror' to false"))))
 
 ;; Apply the Maxima function f to x. If an error is signaled, return nil; otherwise
@@ -1248,7 +1259,9 @@ a positive integer; instead found ~:M" n-sub))))
     (setq x (margs (simplify (apply #'fmapl1 (cons f x)))))
     (checked-and (mapcar #'(lambda (s) ($every '$identity s)) x))))
  
- (t (merror "Improper arguments to function 'every'"))))
+ (t
+   ;; NOT CLEAR FROM PRECEDING CODE WHAT IS "INVALID" HERE
+   (merror (intl:gettext "every: invalid arguments.")))))
 
 (defun $some (f &rest x)
   (cond ((or (null x) (and (null (cdr x)) ($emptyp (first x)))) nil)
@@ -1262,12 +1275,14 @@ a positive integer; instead found ~:M" n-sub))))
     (setq x (margs (simplify (apply #'fmapl1 (cons f x)))))
     (checked-or (mapcar #'(lambda (s) ($some '$identity s)) x))))
 
- (t (merror "Improper arguments to function 'some'"))))
+ (t
+   ;; NOT CLEAR FROM PRECEDING CODE WHAT IS "INVALID" HERE
+   (merror (intl:gettext "some: invalid arguments.")))))
 
 (defun $makeset (f v s)
   (if (or (not ($listp v)) 
 	  (not (every #'(lambda (x) (or ($atom x) ($subvarp x))) (cdr v))))
-      (merror "The second argument to makeset must be a list of atoms or subscripted variables; found ~:M" v))
+      (merror (intl:gettext "makeset: second argument must be a list of atoms or subscripted variables; found: ~:M") v))
   (setq s (require-list-or-set s "$makeset"))
 
   (setq f `((lambda) ,v ,f))
