@@ -10,33 +10,11 @@
 (defmacro get-integrand (fun var)
   `(coerce-float-fun ,fun `((mlist) ,,var)))
 
-(defun lispify-maxima-keyword-options (options &optional valid-keywords)
-  ;; options looks like (((mequal) $opt1 val1) ((mequal) $opt2 val2) ...)
-  ;;
-  ;; Convert to a new list that looks like ($opt1 val1 $opt2 val2 ...)
-  ;;
-  (unless (listp options)
-    (merror "Invalid Maxima keyword options: ~M" options))
-  (when (every #'(lambda (o)
-		   ;; Make sure every option has the right form.
-		   (let ((ok (and (listp o)
-				  (= (length o) 3)
-				  (eq (caar o) 'mequal))))
-		     (unless ok
-		       (merror "Badly formed keyword option: ~M" o))
-		     ok))
-		 options)
-    (mapcan #'(lambda (o)
-	      (destructuring-bind (mequal opt val)
-		  o
-		(declare (ignore mequal))
-		(if (or (null valid-keywords)
-			(member opt valid-keywords))
-		    (flet ((keywordify (x)
-			     (intern (subseq (symbol-name x) 1) :keyword)))
-		      (list (keywordify opt) val))
-		    (merror "Unrecognized keyword: ~M" opt))))
-	  options)))
+(defun float-or-lose (val)
+  (let ((v ($float val)))
+    (if (numberp v)
+	v
+	(error (intl:gettext "~A is not a real floating-point number") val))))
 
 (defun quad-qag (fun var a b key &key
 		 (epsrel 1e-8)
@@ -48,24 +26,24 @@
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
 	 (f (get-integrand fun var)))
     (handler-case
-    (multiple-value-bind (junk z-a z-b z-epsabs z-epsrel z-key result abserr neval ier
-			       z-limit z-lenw last)
-	(slatec:dqag #'(lambda (x)
-			  (float (funcall f x)))
-		      (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		      (let ((v ($float b))) (if (numberp v) v (throw 'error nil)))
-		      (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		      (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-              key
-		      0.0 0.0 0 0
-		      limit lenw 0 iwork work)
-      (declare (ignore junk z-a z-b z-epsabs z-epsrel z-key z-limit z-lenw last))
-      (list '(mlist) result abserr neval ier))
-      (error (e)
-          `(($quad_qag) ,fun ,var ,a ,b ,key
-                        ((mequal) $epsrel ,epsrel)
-                        ((mequal) $epsabs ,epsabs)
-                        ((mequal) $limit ,limit))))))
+	(multiple-value-bind (junk z-a z-b z-epsabs z-epsrel z-key result abserr neval ier
+				   z-limit z-lenw last)
+	    (slatec:dqag #'(lambda (x)
+			     (float (funcall f x)))
+			 (float-or-lose a)
+			 (float-or-lose b)
+			 (float-or-lose epsabs)
+			 (float-or-lose epsrel)
+			 key
+			 0.0 0.0 0 0
+			 limit lenw 0 iwork work)
+	  (declare (ignore junk z-a z-b z-epsabs z-epsrel z-key z-limit z-lenw last))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qag) ,fun ,var ,a ,b ,key
+	  ((mequal) $epsrel ,epsrel)
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit))))))
 
 (defun quad-qags (fun var a b &key
 		  (epsrel 1e-8)
@@ -77,61 +55,88 @@
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
 	 (f (get-integrand fun var)))
     (handler-case
-    (multiple-value-bind (junk z-a z-b z-epsabs z-epsrel result abserr neval ier
-			       z-limit z-lenw last)
-	(slatec:dqags #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float b))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       limit lenw 0 iwork work)
-      (declare (ignore junk z-a z-b z-epsabs z-epsrel z-limit z-lenw last))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qags) ,fun ,var ,a ,b
-                         ((mequal) $epsrel ,epsrel)
-                         ((mequal) $epsabs ,epsabs)
-                         ((mequal) $limit ,limit))))))
+	(multiple-value-bind (junk z-a z-b z-epsabs z-epsrel result abserr neval ier
+				   z-limit z-lenw last)
+	    (slatec:dqags #'(lambda (x)
+			      (float (funcall f x)))
+			  (float-or-lose a)
+			  (float-or-lose b)
+			  (float-or-lose epsabs)
+			  (float-or-lose epsrel)
+			  0.0 0.0 0 0
+			  limit lenw 0 iwork work)
+	  (declare (ignore junk z-a z-b z-epsabs z-epsrel z-limit z-lenw last))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qags) ,fun ,var ,a ,b
+	  ((mequal) $epsrel ,epsrel)
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit))))))
 
-(defun quad-qagi (fun var bound inf-type &key
-		  (epsrel 1e-8)
-		  (limit 200)
-		  (epsabs 0.0))
-  (quad_argument_check fun var bound inf-type) 
-  (let* ((lenw (* 4 limit))
-	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var))
-	 (infinity (case inf-type
-		     ((1 $inf)
-		      ;; Interval is [bound, infinity]
-		      1)
-		     ((-1 $minf)
-		      ;; Interval is [-infinity, bound]
-		      -1)
-		     ((2 $both)
-		      ;; Interval is [-infinity, infinity]
-		      2))))
-    (handler-case
-    (multiple-value-bind (junk z-bound z-inf z-epsabs z-epsrel result abserr neval ier
-			       z-limit z-lenw last)
-	(slatec:dqagi #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float bound))) (if (numberp v) v (throw 'error nil)))
-		       infinity
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       limit lenw 0 iwork work)
-      (declare (ignore junk z-bound z-inf z-epsabs z-epsrel z-limit z-lenw last))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qagi) ,fun ,var ,bound ,inf-type
-                          ((mequal) $epsrel ,epsrel)
-                          ((mequal) $epsabs ,epsabs)
-                          ((mequal) $limit ,limit))))))
+(defun quad-qagi (fun var a b &key
+		      (epsrel 1e-8)
+		      (limit 200)
+		      (epsabs 0.0))
+  (quad_argument_check fun var a b)
+  ;; Massage the limits a and b into what Quadpack QAGI wants.
+  (flet ((fixup (low high)
+	   (let (bnd inf)
+	     ;; Cases to handle: (minf, x), (x, inf), (minf, inf).
+	     ;; Everything else is an error.
+	     (cond ((eq low '$minf)
+		    (cond ((eq high '$inf)
+			   (setf bnd 0)
+			   (setf inf 2))
+			  (t
+			   (setq bnd ($float high))
+			   (setq inf low))))
+		   ((eq high '$inf)
+		    (setq bnd ($float low))
+		    (setq inf high))
+		   (t
+		    (return-from quad-qagi
+		      `(($quad_qagi) ,fun ,var ,a ,b
+			((mequal) $epsrel ,epsrel)
+			((mequal) $epsabs ,epsabs)
+			((mequal) $limit ,limit)))))
+	     (values bnd inf))))
+
+    (multiple-value-bind (bound inf-type)
+	(fixup a b)
+      (let* ((lenw (* 4 limit))
+	     (work (make-array lenw :element-type 'flonum))
+	     (iwork (make-array limit :element-type 'f2cl-lib:integer4))
+	     (f (get-integrand fun var))
+	     (infinity (case inf-type
+			 ((1 $inf)
+			  ;; Interval is [bound, infinity]
+			  1)
+			 ((-1 $minf)
+			  ;; Interval is [-infinity, bound]
+			  -1)
+			 ((2 $both)
+			  ;; Interval is [-infinity, infinity]
+			  2))))
+	(handler-case
+	    (multiple-value-bind (junk z-bound z-inf z-epsabs z-epsrel
+				       result abserr neval ier
+				       z-limit z-lenw last)
+		(slatec:dqagi #'(lambda (x)
+				  (float (funcall f x)))
+			      (float-or-lose bound)
+			      infinity
+			      (float-or-lose epsabs)
+			      (float-or-lose epsrel)
+			      0.0 0.0 0 0
+			      limit lenw 0 iwork work)
+	      (declare (ignore junk z-bound z-inf z-epsabs z-epsrel
+			       z-limit z-lenw last))
+	      (list '(mlist) result abserr neval ier))
+	  (error ()
+	    `(($quad_qagi) ,fun ,var ,a ,b
+	      ((mequal) $epsrel ,epsrel)
+	      ((mequal) $epsabs ,epsabs)
+	      ((mequal) $limit ,limit))))))))
 
 (defun quad-qawc (fun var c a b &key
 		  (epsrel 1e-8)
@@ -143,24 +148,24 @@
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
 	 (f (get-integrand fun var)))
     (handler-case
-    (multiple-value-bind (junk z-a z-b z-c z-epsabs z-epsrel result abserr neval ier
-			       z-limit z-lenw last)
-	(slatec:dqawc #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float b))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float c))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       limit lenw 0 iwork work)
-      (declare (ignore junk z-a z-b z-c z-epsabs z-epsrel z-limit z-lenw last))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qawc) ,fun ,var ,c ,a ,b
-                          ((mequal) $epsrel ,epsrel)
-                          ((mequal) $epsabs ,epsabs)
-                          ((mequal) $limit ,limit))))))
+	(multiple-value-bind (junk z-a z-b z-c z-epsabs z-epsrel result abserr neval ier
+				   z-limit z-lenw last)
+	    (slatec:dqawc #'(lambda (x)
+			      (float (funcall f x)))
+			  (float-or-lose a)
+			  (float-or-lose b)
+			  (float-or-lose c)
+			  (float-or-lose epsabs)
+			  (float-or-lose epsrel)
+			  0.0 0.0 0 0
+			  limit lenw 0 iwork work)
+	  (declare (ignore junk z-a z-b z-c z-epsabs z-epsrel z-limit z-lenw last))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qawc) ,fun ,var ,c ,a ,b
+	  ((mequal) $epsrel ,epsrel)
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit))))))
 
 (defun quad-qawf (fun var a omega trig &key
 		  (epsabs 1e-10)
@@ -176,29 +181,30 @@
 		   ((1 %cos $cos) 1)
 		   ((2 %sin $sin) 2))))
     (handler-case
-    (multiple-value-bind (junk z-a z-omega z-integr epsabs result abserr neval ier
-			       z-limlst z-lst
-			       z-leniw z-maxp1 z-lenw)
-	(slatec:dqawf #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float omega))) (if (numberp v) v (throw 'error nil)))
-		       integr
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       limlst 0 leniw maxp1 lenw iwork work)
-      (declare (ignore junk z-a z-omega z-integr epsabs z-limlst z-lst
-		       z-leniw z-maxp1 z-lenw))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qawf) ,fun ,var ,a ,omega ,trig
-                          ((mequal) $epsabs ,epsabs)
-                          ((mequal) $limit ,limit)
-                          ((mequal) $maxp1 ,maxp1)
-                          ((mequal) $limlst ,limlst))))))
+	(multiple-value-bind (junk z-a z-omega z-integr
+				   epsabs result abserr neval ier
+				   z-limlst z-lst
+				   z-leniw z-maxp1 z-lenw)
+	    (slatec:dqawf #'(lambda (x)
+			      (float (funcall f x)))
+			  (float-or-lose a)
+			  (float-or-lose omega)
+			  integr
+			  (float-or-lose epsabs)
+			  0.0 0.0 0 0
+			  limlst 0 leniw maxp1 lenw iwork work)
+	  (declare (ignore junk z-a z-omega z-integr epsabs z-limlst z-lst
+			   z-leniw z-maxp1 z-lenw))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qawf) ,fun ,var ,a ,omega ,trig
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit)
+	  ((mequal) $maxp1 ,maxp1)
+	  ((mequal) $limlst ,limlst))))))
 
 (defun quad-qawo (fun var a b omega trig &key
-		  (epsrel 1e-10)
+		  (epsrel 1e-8)
 		  (limit 200)
 		  (maxp1 100)
 		  (epsabs 0.0))
@@ -212,31 +218,31 @@
 		   ((1 %cos $cos) 1)
 		   ((2 %sin $sin) 2))))
     (handler-case
-    (multiple-value-bind (junk z-a z-b z-omega z-integr z-epsabs z-epsrel
-			       result abserr neval ier
-			       z-leniw z-maxp1 z-lenw z-lst)
-	(slatec:dqawo #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float b))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float omega))) (if (numberp v) v (throw 'error nil)))
-		       integr
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       leniw maxp1 lenw 0 iwork work)
-      (declare (ignore junk z-a z-b z-omega z-integr z-epsabs z-epsrel
-		       z-lst z-leniw z-maxp1 z-lenw))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qawo) ,fun ,var ,a ,b ,omega ,trig
-                          ((mequal) $epsrel ,epsrel)
-                          ((mequal) $epsabs ,epsabs)
-                          ((mequal) $limit ,limit)
-                          ((mequal) $maxp1 ,maxp1))))))
+	(multiple-value-bind (junk z-a z-b z-omega z-integr z-epsabs z-epsrel
+				   result abserr neval ier
+				   z-leniw z-maxp1 z-lenw z-lst)
+	    (slatec:dqawo #'(lambda (x)
+			      (float (funcall f x)))
+			  (float-or-lose a)
+			  (float-or-lose b)
+			  (float-or-lose omega)
+			  integr
+			  (float-or-lose epsabs)
+			  (float-or-lose epsrel)
+			  0.0 0.0 0 0
+			  leniw maxp1 lenw 0 iwork work)
+	  (declare (ignore junk z-a z-b z-omega z-integr z-epsabs z-epsrel
+			   z-lst z-leniw z-maxp1 z-lenw))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qawo) ,fun ,var ,a ,b ,omega ,trig
+	  ((mequal) $epsrel ,epsrel)
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit)
+	  ((mequal) $maxp1 ,maxp1))))))
 
 (defun quad-qaws (fun var a b alfa beta wfun &key
-		  (epsrel 1e-10)
+		  (epsrel 1e-8)
 		  (limit 200)
 		  (epsabs 0.0))
   (quad_argument_check fun var a b) 
@@ -245,28 +251,28 @@
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
 	 (f (get-integrand fun var)))
     (handler-case
-    (multiple-value-bind (junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
-			       result abserr neval ier
-			       z-limit z-lenw last)
-	(slatec:dqaws #'(lambda (x)
-			   (float (funcall f x)))
-		       (let ((v ($float a))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float b))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float alfa))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float beta))) (if (numberp v) v (throw 'error nil)))
-		       wfun
-		       (let ((v ($float epsabs))) (if (numberp v) v (throw 'error nil)))
-		       (let ((v ($float epsrel))) (if (numberp v) v (throw 'error nil)))
-		       0.0 0.0 0 0
-		       limit lenw 0 iwork work)
-      (declare (ignore junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
-		       z-limit z-lenw last))
-      (list '(mlist) result abserr neval ier))
-    (error (e)
-           `(($quad_qaws) ,fun ,var ,a ,b ,alfa ,beta ,wfun
-                          ((mequal) $epsrel ,epsrel)
-                          ((mequal) $epsabs ,epsabs)
-                          ((mequal) $limit ,limit))))))
+	(multiple-value-bind (junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
+				   result abserr neval ier
+				   z-limit z-lenw last)
+	    (slatec:dqaws #'(lambda (x)
+			      (float (funcall f x)))
+			  (float-or-lose a)
+			  (float-or-lose b)
+			  (float-or-lose alfa)
+			  (float-or-lose beta)
+			  wfun
+			  (float-or-lose epsabs)
+			  (float-or-lose epsrel)
+			  0.0 0.0 0 0
+			  limit lenw 0 iwork work)
+	  (declare (ignore junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
+			   z-limit z-lenw last))
+	  (list '(mlist) result abserr neval ier))
+      (error ()
+	`(($quad_qaws) ,fun ,var ,a ,b ,alfa ,beta ,wfun
+	  ((mequal) $epsrel ,epsrel)
+	  ((mequal) $epsabs ,epsabs)
+	  ((mequal) $limit ,limit))))))
 
 ;; error checking similar to that done by $defint
 (defun quad_argument_check (exp var ll ul) 
@@ -289,13 +295,14 @@
 	      (options (gensym "OPTIONS-")))
 	 `(defun ,mname (,@args &rest ,options)
 	    (let
-          ((,keylist (lispify-maxima-keyword-options ,options ',valid-keys))
-           ;; BIND EVIL SPECIAL VARIABLE *PLOT-REALPART* HERE ...
-           (*plot-realpart* nil))
+		((,keylist (lispify-maxima-keyword-options ,options ',valid-keys))
+		 ;; BIND EVIL SPECIAL VARIABLE *PLOT-REALPART* HERE ...
+		 (*plot-realpart* nil))
+	      (declare (special *plot-realpart*))
 	      (apply ',iname ,@args ,keylist))))))
   (frob $quad_qag quad-qag (fun var a b key) ($epsrel $limit $epsabs))
   (frob $quad_qags quad-qags (fun var a b) ($epsrel $limit $epsabs))
-  (frob $quad_qagi quad-qagi (fun var bound inf-type) ($epsrel $limit $epsabs))
+  (frob $quad_qagi quad-qagi (fun var a b) ($epsrel $limit $epsabs))
   (frob $quad_qawc quad-qawc (fun var c a b) ($epsrel $limit $epsabs))
   (frob $quad_qawf quad-qawf (fun var a omega trig) ($limit $epsabs $maxp1 $limlst))
   (frob $quad_qawo quad-qawo (fun var a b omega trig) ($epsrel $limit $epsabs $maxp1))
