@@ -14,16 +14,16 @@
 
 (load-macsyma-macros rzmac ratmac)
 
-(declare-top (special prob rootfac parnumer pardenom logptdx wholepart $ratalgdenom
-		     expexpflag $logsimp switch1 degree cary $ratfac $logexpand
-		     ratform genvar *var var rootfactor expint $keepfloat
-		     trigint operator $exponentialize $gcd $logarc changevp
-		     klth r s beta gamma b mainvar expflag expstuff liflag
-		     intvar switch varlist nogood genvar $erfflag $liflag
-		     rischp $factorflag alphar m simp genpairs hypertrigint
-		     *mosesflag yyy *exp y $algebraic implicit-real
-		     errrjfflag $%e_to_numlog generate-atan2 context
-		     bigfloatzero rp-polylogp))
+(declare-top (special prob rootfac parnumer pardenom logptdx wholepart
+                      $ratalgdenom expexpflag $logsimp switch1 degree cary
+                      $ratfac $logexpand ratform genvar *var var rootfactor
+                      expint $keepfloat trigint operator $exponentialize $gcd
+                      $logarc changevp klth r s beta gamma b mainvar expflag
+                      expstuff liflag intvar switch varlist nogood genvar
+                      $erfflag $liflag rischp $factorflag alphar m simp
+                      genpairs hypertrigint *mosesflag yyy *exp y $algebraic
+                      implicit-real errrjfflag $%e_to_numlog generate-atan2
+                      context bigfloatzero rp-polylogp *in-risch-p*))
 
 (defmvar $liflag t "Controls whether `risch' generates polylogs")
 
@@ -47,6 +47,10 @@
        (gl genvar (cdr gl)))
       ((null (cdr vl)) (car gl))))
 
+;; test whether CRE p is constant with respect to variable of integration.
+;; requires variables in varlist and genvar
+;; to be ordered as by intsetup, with var of integration ordered before
+;; any other expressions that contain it.
 (defun risch-pconstp (p)
   (or (pcoefp p) (pointergp mainvar (car p))))
 
@@ -102,9 +106,12 @@
   (and (mqapplyp exp) (eq (subfunname exp) '$li)
        (or (null sub) (equal sub (car (subfunsubs exp))))))
 
-
 (defun rischint (exp intvar &aux ($logarc nil) ($exponentialize nil)
-		 ($gcd '$algebraic) ($algebraic t) (implicit-real t))
+                     ($gcd '$algebraic) ($algebraic t) (implicit-real t)
+                     ;; The risch integrator expects $logexpand T. Otherwise,
+                     ;; the integrator hangs for special types of integrals
+                     ;; (See bug report ID:3039452)
+                     ($logexpand t))
   (prog ($%e_to_numlog $logsimp trigint operator y z var ratform liflag
 	 mainvar varlist genvar hypertrigint $ratfac $ratalgdenom )
      (if (specrepp exp) (setq exp (specdisrep exp)))
@@ -163,18 +170,34 @@
 	(t (setq operator (caar l)))))
 
 (defun hypertrigint1 (exp var hyperfunc)
-  (if hyperfunc (integrator (resimplify exp) var)
-      (rischint (resimplify exp) var)))
+  (let ((result (if hyperfunc
+                    (integrator (resimplify exp) var)
+                    (rischint (resimplify exp) var))))
+    ;; The result can contain solveable integrals. Look for this case.
+    (if (isinop result '%integrate)
+        ;; Found an integral. Evaluate the result again.
+        ;; Set the flag *in-risch-p* to make sure that we do not call
+        ;; rischint again from the integrator. This avoids endless loops.
+        (let ((*in-risch-p* t)) 
+          (meval (list '($ev) result '$nouns)))
+        result)))
 
 (defun trigin1 (*exp var)
   (let ((yyy (hypertrigint1 *exp var nil)))
     (setq yyy (div ($expand ($num yyy))
 		   ($expand ($denom yyy))))
-    (let ((rischp var) (rp-polylogp t) $logarc $exponentialize)
-      (sratsimp (if (and (freeof '$%i *exp) (freeof '$li yyy))
-		    ($realpart yyy)
-		    ($rectform yyy))))))
-
+    (let ((rischp var) (rp-polylogp t) $logarc $exponentialize result)
+      (setq result (sratsimp (if (and (freeof '$%i *exp) (freeof '$li yyy))
+                                 ($realpart yyy)
+                                 ($rectform yyy))))
+      ;; The result can contain solveable integrals. Look for this case.
+      (if (isinop result '%integrate)
+          ;; Found an integral. Evaluate the result again.
+          ;; Set the flag *in-risch-p* to make sure that we do not call
+          ;; rischint again from the integrator. This avoids endless loops.
+          (let ((*in-risch-p* t)) 
+            (meval (list '($ev) result '$nouns)))
+          result))))
 
 (defun tryrisch (exp mainvar)
   (prog (wholepart rootfactor parnumer pardenom
@@ -347,6 +370,18 @@
 				      (getfncoeff (cdr a) form)))
 	((eq (caaar a) 'mtimes)
 	 (destructuring-let (((coef . newfn) (getfnsplit (cdar a))))
+           ;; (car a) is a mtimes expression. We insert coef and newfn as the
+           ;; new arguments to the mtimes expression. This causes problems if
+           ;;   (1) coef is a mtimes expression too and
+           ;;   (2) (car a) has already a simp flag
+           ;; We get a nested mtimes expression, which does not simplify.
+	   ;; We comment out the following code (DK 09/2009):
+           ;; (setf (cdar a) (list coef newfn))
+	   
+	   ;; Insert a complete mtimes expression without simpflag.
+           ;; Nested mtimes expressions simplify further.
+           (setf (car a) (list '(mtimes) coef newfn))
+	   
 	   (setf (cdar a) (list coef newfn))
 	   (cond ((zerop1 coef) (getfncoeff (cdr a) form))
 		 ((and (matanp newfn) (member '$%i varlist :test #'eq))
@@ -440,7 +475,7 @@
 		       (narg (remabs (cadr nlog)))
 		       (varlist varlist)
 		       (genvar genvar)
-		       (rn (rform narg))
+		       (rn (rform narg))	;; can add new vars to varlist
 		       (ro (rform (cadr olog)))
 		       (var (caar ro))
 		       ((j . k) (ratreduce (pdegree (car rn) var) (pdegree (car ro) var)))
@@ -455,8 +490,8 @@
 		 coef (div coef (f* j degree))
 		 olog (mul j olog))))
     (desetq (rc . rd) (ratdivide rn ro))
-    (cond ((and (risch-constp rc)
-		(risch-constp rd))
+    (cond ((and (freeof intvar (rdis rc))	;; can't use risch-constp because varlist
+		(freeof intvar (rdis rd)))	;; is not set up with vars in correct order.
 	   (setq narg ($ratsimp (sub 1 (div narg (rdis rd)))))
 	   (mul* coef (power -1 (1+ degree))
 		 `((mfactorial) ,degree)
