@@ -17,7 +17,7 @@
 (declare-top (special var-list expsumsplit $dispflag checkfactors *g
 		      $algebraic equations ;List of E-labels
 		      *power *varb *flg $derivsubst $numer $float
-		      $%emode wflag genvar genpairs varlist broken-not-freeof
+		      $%emode genvar genpairs varlist broken-not-freeof
 		      $factorflag
 		      mult    ;Some crock which tracks multiplicities.
 		      *roots ;alternating list of solutions and multiplicities
@@ -39,10 +39,6 @@
 (defmvar $linsolvewarn t
   "Needs to be documented.")
 
-(defmvar $solve_inconsistent_error t
-  "If T gives an MAXIMA-ERROR if SOLVE meets up with inconsistent linear
-	 equations.  If NIL, returns ((MLIST SIMP)) in this case.")
-
 (defmvar $programmode t
   "Causes SOLVE to return its answers explicitly as elements
 	 in a list rather than printing E-labels.")
@@ -138,10 +134,10 @@
 
      ;; Some sanity checks and warning messages.
      (when (and (null varl) $solvenullwarn)
-       (mtell "~&Got a null variable list, continuing - `solve'~%"))
+       (mtell "~&solve: variable list is empty, continuing anyway.~%"))
 
      (when (and (null eql) $solvenullwarn)
-       (mtell "~&Got a null equation list, continuing - `solve'~%"))
+       (mtell "~&solve: equation list is empty, continuing anyway.~%"))
 
      (when (some #'mnump varl)
        (merror
@@ -224,10 +220,10 @@
 							   (nconc *roots *failures)))))
 		      (setq $multiplicities (make-mlist-l (nreverse multi)))))
 		   (t (when (and *failures (not $solveexplicit))
-			(when $dispflag (mtell "The roots of:~%"))
+			(when $dispflag (mtell "solve: the roots of:~%"))
 			(solve2 *failures))
 		      (when *roots
-			(when $dispflag (mtell "Solution:~%"))
+			(when $dispflag (mtell "solve: solution:~%"))
 			(solve2 *roots))
 		      (make-mlist-l equations)))))))
 
@@ -274,7 +270,7 @@
 		(cond ($solveradcan (setq exp (radcan1 exp))
 				    (if (atom exp) (go a))))
 	      
-		(cond ((easy-cases exp *var)
+		(cond ((easy-cases exp *var mult)
 		       (cond (symbol (setq *roots (subst temp *var *roots))
 				     (setq *failures (subst temp *var *failures))))
 		       (rootsort *roots)
@@ -334,24 +330,27 @@
 ;;; Solve is not fully recursive when it due to globals, $MULTIPLICIES
 ;;; may be screwed here. (Solve should be made recursive)
 
-(defun easy-cases (*exp *var)
+(defun easy-cases (*exp *var mult)
   (cond ((or (atom *exp) (atom (car *exp))) nil)
 	((eq (caar *exp) 'mtimes)
 	 (do ((terms (cdr *exp) (cdr terms)))
 	     ((null terms))
-	   (solve (car terms) *var 1))
-	 'mtimes)))
+	   (solve (car terms) *var mult))
+	 'mtimes)
 
-;; This code is commented out because it exposes a bug in the way
-;; solve (or its friends) handles multiplicities. A previous 
-;; version (1.2) had a typo (caar *exp) 'mexp ...) that prevented this
-;; bug from manifesting.  Barton Willis, 12 May 2004
-
-;;	     ((EQ (CAAR *EXP) 'MEXPT)
-;;	      (COND ((AND (INTEGERP  (CADDR *EXP))
-;;			  (PLUSP (CADDR *EXP)))
-;;		     (SOLVE (CADR *EXP) *VAR (CADDR *EXP))
-;;		     'MEXPRAT)))))
+	;; This code is commented out because it exposes a bug in the way
+	;; solve (or its friends) handles multiplicities. A previous 
+	;; version (1.2) had a typo (caar *exp) 'mexp ...) that prevented this
+	;; bug from manifesting.  Barton Willis, 12 May 2004
+	;;
+	;; In particular it causes test 137 in rtest15 and test 45 in
+	;; rtestint to fail.
+	#+nil
+	((eq (caar *exp) 'mexpt)
+	 (cond ((and (integerp  (caddr *exp))
+		     (plusp (caddr *exp)))
+		(solve (cadr *exp) *var (caddr *exp))
+		'mexprat)))))
 
 ;;; Predicate to test for presence of troublesome trig functions to be
 ;;; canonicalized.  A  table of when to make substitutions should
@@ -562,9 +561,9 @@
   (let ((*myvar *myvar)
 	(*g nil)) 
     (cond ((atom exp) nil)
-	  ((not (memalike (setq *myvar (pdis (list (car exp) 1 1)))
-			  *has*var))
-	   nil)
+          ((not (memalike (setq *myvar (simplify (pdis (list (car exp) 1 1))))
+                          *has*var))
+           nil)
 	  ((equal (cadr exp) 1) (solvelin exp))
 	  ((of-form-A*F<X>^N+B exp) (solve-A*F<X>^N+B exp t))
 	  ((equal (cadr exp) 2) (solvequad exp))
@@ -702,10 +701,10 @@
 ;; Sees if expression is of the form A*F<X>^N+B.
 
 (defun of-form-A*F<X>^N+B (e)
-  (and (memalike (pdis (list (car e) 1 1)) *has*var)
+  (and (memalike (simplify (pdis (list (car e) 1 1))) *has*var)
        (or (atom (caddr e))
-	   (not (memalike (pdis (list (caaddr e) 1 1))
-			  *has*var)))
+           (not (memalike (simplify (pdis (list (caaddr e) 1 1)))
+                          *has*var)))
        (or (null (cdddr e)) (equal (cadddr e) 0))))
 
 ;; Solves the special case A*F<X>^N+B.
@@ -820,7 +819,7 @@
 		     (setq *v (delete z *v :count 1 :test #'equal)))))
 	  v)
     (setq $dontfactor *u)
-    (setq *has*var *v)
+    (setq *has*var (mapcar #'resimplify *v))
     (append *u *v)))
 
 ;; Solves for variable when it occurs within a function by taking the inverse.
@@ -869,8 +868,7 @@
 	     ((setq inverse (get op '$inverse))
 	      (when (and $solvetrigwarn
 			 (member op '(%sin %cos %tan %sec %csc %cot %cosh %sech) :test #'eq))
-		(mtell "~&`solve' is using arc-trig functions to get ~
-			    a solution.~%Some solutions will be lost.~%")
+		(mtell "~&solve: using arc-trig functions to get a solution.~%Some solutions will be lost.~%")
 		(setq $solvetrigwarn nil))
 	      `((mplus) ((mminus) ,(cadr *myvar))
 		((,inverse) ,exp)))
@@ -936,11 +934,9 @@
 		  (t (merror "`linsolve' ran into a nonlinear equation.")))))
      (setq ans (tfgeli 'xa* xn* xm*))
      (if (and $linsolvewarn (car ans))
-	 (mtell "~&Dependent equations eliminated:  ~A~%" (car ans)))
+	 (mtell "~&solve: dependent equations eliminated: ~A~%" (car ans)))
      (if (cadr ans)
-	 (if $solve_inconsistent_error
-	     (merror "Inconsistent equations:  ~M" (cons '(mlist) (cadr ans)))
-	     (return '((mlist simp)))))
+	 (return '((mlist simp))))
      (do ((j 0 (1+ j)))
 	 ((> j xm*))
        ;;I put this in the value cell--wfs 
