@@ -18,7 +18,7 @@
 
 (defmvar $%piargs t)
 (defmvar $%iargs t)
-(defmvar $triginverses '$all)
+(defmvar $triginverses t)
 (defmvar $trigexpand nil)
 (defmvar $trigexpandplus t)
 (defmvar $trigexpandtimes t)
@@ -27,18 +27,17 @@
 (defmvar $logarc nil)
 (defmvar $halfangles nil)
 
-(defmvar 1//2 '((rat simp) 1 2))
-(defmvar -1//2 '((rat simp) -1 2))
-(defmvar %pi//4 '((mtimes simp) ((rat simp) 1 4.) $%pi))
-(defmvar %pi//2 '((mtimes simp) ((rat simp) 1 2) $%pi))
-(defmvar sqrt2//2 '((mtimes simp) ((rat simp) 1 2)
-		    ((mexpt simp) 2 ((rat simp) 1 2))))
-(defmvar -sqrt2//2 '((mtimes simp) ((rat simp) -1 2)
-		     ((mexpt simp) 2 ((rat simp) 1 2))))
-(defmvar sqrt3//2 '((mtimes simp) ((rat simp) 1 2)
-		    ((mexpt simp) 3 ((rat simp) 1 2))))
-(defmvar -sqrt3//2 '((mtimes simp) ((rat simp) -1 2)
-		     ((mexpt simp) 3 ((rat simp) 1 2))))
+;; Simplified shortcuts for constant expressions.
+(defvar 1//2 '((rat simp) 1 2))
+(defvar -1//2 '((rat simp) -1 2))
+(defvar %pi//4 '((mtimes simp) ((rat simp) 1 4.) $%pi))
+(defvar %pi//2 '((mtimes simp) ((rat simp) 1 2) $%pi))
+(defvar sqrt3//2 '((mtimes simp)
+                   ((rat simp) 1 2) 
+                   ((mexpt simp) 3 ((rat simp) 1 2))))
+(defvar -sqrt3//2 '((mtimes simp)
+                    ((rat simp) -1 2)
+                    ((mexpt simp) 3 ((rat simp) 1 2))))
 
 ;;; Arithmetic utilities.
 
@@ -87,6 +86,14 @@
 (defprop %acsch simp-%acsch operators)
 (defprop %asech simp-%asech operators)
 
+;;; The trigonometric functions distribute of lists, matrices and equations.
+
+(dolist (x '(%sin   %cos   %tan   %cot   %csc   %sec
+             %sinh  %cosh  %tanh  %coth  %csch  %sech
+             %asin  %acos  %atan  %acot  %acsc  %asec
+             %asinh %acosh %atanh %acoth %acsch %asech))
+  (setf (get x 'distribute_over) '(mlist $matrix mequal)))
+
 (defun domain-error (x f)
   (merror "The number ~:M isn't in the domain of ~A" (complexify x) f))
 
@@ -176,6 +183,12 @@
     (- #.(/ (float pi) 2) (maxima-branch-asin x))
     (cl:acos x)))
 
+(defun maxima-branch-acot (x)
+  ;; Allow 0.0 in domain of acot, otherwise use atan(1/x)
+  (if (and (equal (realpart x) 0.0) (equal (imagpart x) 0.0))
+    #.(/ (float pi) 2)
+    (cl:atan (/ 1 x))))
+
 ;; Apply formula from CLHS if X falls on a branch cut.
 ;; Otherwise punt to CL:ATANH.
 (defun maxima-branch-atanh (x)
@@ -221,7 +234,7 @@
 		    (if y y (domain-error x 'acsc)))))
 
   (frob %acot #'(lambda (x)
-		  (let ((y (ignore-errors (cl:atan (/ 1 x)))))
+		  (let ((y (ignore-errors (maxima-branch-acot x))))
 		    (if y y (domain-error x 'acot)))))
 
   (frob %cosh #'cl:cosh)
@@ -257,8 +270,8 @@
 		   (let ((y (ignore-errors (maxima-branch-atanh (/ 1 x))))) 
 		     (if y y (domain-error x 'acoth)))))
 
-  (frob %mabs #'cl:abs)
-  (frob $exp #'cl:exp)
+  (frob mabs #'cl:abs)
+  (frob %exp #'cl:exp)
   (frob mexpt #'cl:expt)
   (frob %sqrt #'cl:sqrt)
   (frob %log #'(lambda (x)
@@ -276,7 +289,7 @@
   (frob $imagpart #'cl:imagpart)
   (frob $max #'cl:max)
   (frob $min #'cl:min)
-  (frob %signnum #'cl:signum)
+  (frob %signum #'cl:signum)
   (frob $atan2 #'cl:atan)
   (frob %log #'(lambda (x)
 		 (let ((y (ignore-errors (cl:log x))))
@@ -676,52 +689,71 @@
 		   ((alike1 1//2 x) (cons-exp '%sec zl-rem))
 		   ((alike1 '((rat) 3 2) x) (neg (cons-exp '%sec zl-rem)))))))
 
-(defmfun simp-%atan (form y z)
+(defun simp-%atan (form y z)
   (oneargcheck form)
   (setq y (simpcheck (cadr form) z))
   (cond ((flonum-eval (mop form) y))
-	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
-	((taylorize (mop form) (second form)))
-	((and $%piargs
-	      ;; Recognize some special values
-	      (cond ((zerop1 y)
-		     0)
-		    ((equal 1 y)
-		     %pi//4)
-		    ((equal -1 y)
-		     (neg %pi//4))
-		    ;; sqrt(3)
-		    ((alike1 y (power* 3 1//2))
-		     (div '$%pi 3))
-		    ;; -sqrt(3)
-		    ((alike1 y (mul -1 (power* 3 1//2)))
-		     (div '$%pi -3))
-		    ;; 1/sqrt(3) = sqrt(3)/3
-		    ((or (alike1 y (power* 3 -1//2))
-			 (alike1 y (mul (rat 1 3) (power* 3 1//2))))
-		     (div '$%pi 6))
-		    ;; -1/sqrt(3) = -sqrt(3)/3
-		    ((or (alike1 y (mul -1 (power* 3 -1//2)))
-			 (alike1 y (div (power* 3 1//2) -3)))
-		     (div '$%pi -6)))))
-	((and $%iargs (multiplep y '$%i)) (mul '$%i (cons-exp '%atanh (coeff y '$%i 1))))
+        ((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
+        ((taylorize (mop form) (second form)))
+        ;; Simplification for special values
+        ((zerop1 y) y)
+        ((or (eq y '$inf) (alike1 y '((mtimes) -1 $minf)))
+         (div '$%pi 2))
+        ((or (eq y '$minf) (alike1 y '((mtimes) -1 $inf)))
+         (div '$%pi -2))
+        ((and $%piargs
+              ;; Recognize more special values
+              (cond ((equal 1 y) (div '$%pi 4))
+                    ((equal -1 y) (div '$%pi -4))
+                    ;; sqrt(3)
+                    ((alike1 y '((mexpt) 3 ((rat) 1 2)))
+                     (div '$%pi 3))
+                    ;; -sqrt(3)
+                    ((alike1 y '((mtimes) -1 ((mexpt) 3 ((rat) 1 2))))
+                     (div '$%pi -3))
+                    ;; 1/sqrt(3)
+                    ((alike1 y '((mexpt) 3 ((rat) -1 2)))
+                     (div '$%pi 6))
+                    ;; -1/sqrt(3)
+                    ((alike1 y '((mtimes) -1 ((mexpt) 3 ((rat) -1 2))))
+                     (div '$%pi -6))
+                    ((alike1 y '((mplus) -1 ((mexpt) 2 ((rat) 1 2))))
+                     (div '$%pi 8))
+                    ((alike1 y '((mplus) 1 ((mexpt) 2 ((rat) 1 2))))
+                     (mul 3 (div '$%pi 8))))))
+        ((and $%iargs (multiplep y '$%i))
+         ;; atan(%i*y) -> %i*atanh(y)
+         (mul '$%i (take '(%atanh) (coeff y '$%i 1))))
+	((and (not (atom y)) (member (caar y) '(%cot %tan))
+	      (if ($constantp (cadr y))
+		  (let ((y-val (mfuncall '$mod 
+					 (if (eq (caar y) '%tan)
+					     (cadr y)
+					     (sub %pi//2 (cadr y)))
+					 '$%pi)))
+		    (cond ((eq (mlsp y-val %pi//2) t) y-val)
+			  ((eq (mlsp y-val '$%pi) t) (sub y-val '$%pi)))))))
 	((and (eq $triginverses '$all) (not (atom y))
 	      (if (eq (caar y) '%tan) (cadr y))))
+	((and (eq $triginverses t) (not (atom y)) (eq (caar y) '%tan)
+	      ;; Check if y in [-%pi/2, %pi/2]
+	      (if (and (member (csign (sub (cadr y) %pi//2)) '($nz $neg) :test #'eq)
+		       (member (csign (add (cadr y) %pi//2)) '($pz $pos) :test #'eq))
+		  (cadr y))))
 	($logarc (logarc '%atan y))
 	((apply-reflection-simp (mop form) y $trigsign))
-	;((and $trigsign (mminusp* y)) (neg (cons-exp '%atan (neg y))))
 	(t (eqtest (list '(%atan) y) form))))
 
 (defun %piargs (x ratcoeff)
   (let (offset-result)
     (cond ((and (integerp (car x)) (integerp (cdr x))) 0)
-		   ((not (mevenp (car x))) 
-		    (cond ((null ratcoeff) nil)
-		 		  ((and (integerp (car x)) 
-		 		        (setq offset-result (%piargs-offset (cdr x))))
-		 		   (mul (power -1 (sub ratcoeff (cdr x)))
-		 		        offset-result))))
-		   ((%piargs-offset (mmod (cdr x) 2))))))
+	  ((not (mevenp (car x))) 
+	   (cond ((null ratcoeff) nil)
+		 ((and (integerp (car x)) 
+		       (setq offset-result (%piargs-offset (cdr x))))
+		  (mul (power -1 (sub ratcoeff (cdr x)))
+		       offset-result))))
+	  ((%piargs-offset (mmod (cdr x) 2))))))
 
 ; simplifies sin(%pi * x) where x is between 0 and 1 
 ; returns nil if can't simplify
